#include "togl.h"

#include "body.h"
#include "list.h"
#include "track.h"
#include "view.h"	// for BodyDelete_Cmd

//================================================================================================

char *partName[] = {
	"torso", "pelvis", "neck", "face", "hair",

	"left:thigh", "left:leg", "left:foot", "left:toe",
	"left:arm", "left:forearm", "left:hand",

	"right:thigh", "right:leg", "right:foot", "right:toe",
	"right:arm", "right:forearm", "right:hand" };

// related with jointEnum in body.h
char *jointName[] = {
	"torso", "pelvis", "neck", "head", "head",

	"left:hip", "left:knee", "left:ankle", "left:foot",
	"left:shoulder", "left:elbow", "left:wrist",

	"right:hip", "right:knee", "right:ankle", "right:foot",
	"right:shoulder", "right:elbow", "right:wrist",

	// interface with routine 'is key'
	"cog:position", "cog:altitude", "cog:rotX", "cog:rotZ", "cog:orientation" , 
	"notes", "includednotes", 0};

char *trackName[] = {
	"pelvis:X", "pelvis:Y", "pelvis:Z",
	"torso:X", "torso:Y", "torso:Z",
	"head:X", "head:Y", "head:Z",
	// left
	"left:shoulder:X", "left:shoulder:Y", "left:shoulder:Z", "left:shoulder:TY",
	"left:elbow:X", "left:elbow:Y",
	"left:wrist:X", "left:wrist:Y", "left:wrist:Z",
	"left:hip:X", "left:hip:Y", "left:hip:Z",
	"left:knee:X",
	"left:ankle:X", "left:ankle:Y", "left:ankle:Z",
	"left:foot:X",
	// right
	"right:shoulder:X", "right:shoulder:Y", "right:shoulder:Z", "right:shoulder:TY",
	"right:elbow:X", "right:elbow:Y",
	"right:wrist:X", "right:wrist:Y", "right:wrist:Z",
	"right:hip:X", "right:hip:Y", "right:hip:Z",
	"right:knee:X",
	"right:ankle:X", "right:ankle:Y", "right:ankle:Z",
	"right:foot:X",
	// position
	"cog:X", "cog:Y", "cog:Z", "cog:rX", "cog:rY", "cog:rZ",
	// for the interface with viewTrack & Tcl
	"nil", 0};


//================================================================================================


int Body_ObjCmd(ClientData clientData, Tcl_Interp *interp,
					   int objc, Tcl_Obj *CONST objv[])
{
Body *body = (class Body *) clientData;
Tcl_Obj *args[MAX_ARGS];
int index, nb_arg;
pfnObj op;

// setup for use from objects/parts
extern Tcl_Interp *currentInterp;
	currentInterp = interp;

static char *optionsPtr[] = {
	"set", "cset", "get", "limits", "position",
	"shade", "wire", "getshaded", "isshaded",
	"select", "unselect",
	"setcolor", "getcolor", 0};
static pfnObj actionPtr[] = {
	0, 0, 0, 0, 0,
	&Object::Shade, &Object::Wireframe, &Object::GetShaded, &Object::IsShaded,
	&Object::Select, &Object::Unselect,
	&Object::SetColor, &Object::GetColor};
static int nbArg[] = {
	0, 0, 0, 0, 0,
	0, 0, 0, 0,
	0, 0,
	1, 0 };


    if (objc < 3) {
        Tcl_WrongNumArgs(interp, 1, objv, "option arg ?arg ...?");
        return TCL_ERROR;
    }


    if (Tcl_GetIndexFromObj(interp, objv[1], optionsPtr, "option", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

	switch (index) {
		case 0:		body->constraint = FALSE; return body->Set(interp, objc, objv);
		case 1:		body->constraint = TRUE;  return body->Set(interp, objc, objv);
		case 2:		return body->Get(interp, objc, objv);
		case 3:		return body->Limits(interp, objc, objv);
		case 4:		return body->Position(interp, objc, objv);

		default:	op = actionPtr[index];
					nb_arg = nbArg[index];
	}


static char *partPtr[] = { "all", "torso", "pelvis", "body",
						   "neck", "face", "hair", "head", "Head",
						   "left:Arm", "left:arm", "left:forearm", "left:hand",
						   "left:Leg", "left:thigh", "left:leg",
						   "left:Foot", "left:foot", "left:toe",
						   "right:Arm", "right:arm", "right:forearm", "right:hand",
						   "right:Leg", "right:thigh", "right:leg",
						   "right:Foot", "right:foot", "right:toe", 0};
enum {	all, to, pe, bo,
		ne, fa, ha, he, He,
		lA, la, lfa, lh,
		lL, lt, ll,
		lF, lfo, lto,
		rA, ra, rfa, rh,
		rL, rt, rl,
		rF, rfo, rto } index2;

	Tcl_ResetResult(interp);

	for(int a=2; a<objc; a++) {
		if (Tcl_GetIndexFromObj(interp, objv[a], partPtr, "part", 0, (int *) &index2) != TCL_OK) {
			return TCL_ERROR;
		}

		for(int i=0; i<nb_arg; i++) {
			if (a+1>=objc) {
				Tcl_SetResult(interp, "too few arguments : ", TCL_STATIC);
				Tcl_AppendResult(interp, optionsPtr[index], NULL);
				return TCL_ERROR;
			}
			args[i] = objv[++a];
		}			

		switch (index2) {
			case all:	body->DoOperation(op, args); break;
			case to:	body->torso.DoOperation(op, args); break;
			case pe:	body->pelvis.DoOperation(op, args); break;
			case bo:	body->torso.DoOperation(op, args);
						body->pelvis.DoOperation(op, args); break;

			case ne:	body->neck.DoOperation(op, args); break;
			case fa:	body->headFace.DoOperation(op, args); break;
			case ha:	body->headHair.DoOperation(op, args); break;
			case he:	body->neck.DoOperation(op, args); 
						body->headFace.DoOperation(op, args); break;
			case He:	body->neck.DoOperation(op, args);
						body->headFace.DoOperation(op, args); 
						body->headHair.DoOperation(op, args); break;

			case lA:	body->Larm.DoOperation(op, args); break;
			case la:	body->Larm.arm.DoOperation(op, args); break;
			case lfa:	body->Larm.forearm.DoOperation(op, args); break;
			case lh:	body->Larm.hand.DoOperation(op, args); break;
			case lL:	body->Lleg.DoOperation(op, args); break;
			case lt:	body->Lleg.thigh.DoOperation(op, args); break;
			case ll:	body->Lleg.leg.DoOperation(op, args); break;
			case lF:	body->Lleg.foot.DoOperation(op, args);
						body->Lleg.toe.DoOperation(op, args); break;
			case lfo:	body->Lleg.foot.DoOperation(op, args); break;
			case lto:	body->Lleg.toe.DoOperation(op, args); break;

			case rA:	body->Rarm.DoOperation(op, args); break;
			case ra:	body->Rarm.arm.DoOperation(op, args); break;
			case rfa:	body->Rarm.forearm.DoOperation(op, args); break;
			case rh:	body->Rarm.hand.DoOperation(op, args); break;
			case rL:	body->Rleg.DoOperation(op, args); break;
			case rt:	body->Rleg.thigh.DoOperation(op, args); break;
			case rl:	body->Rleg.leg.DoOperation(op, args); break;
			case rF:	body->Rleg.foot.DoOperation(op, args);
						body->Rleg.toe.DoOperation(op, args); break;
			case rfo:	body->Rleg.foot.DoOperation(op, args); break;
			case rto:	body->Rleg.toe.DoOperation(op, args); break;
		}
	}

	// force the redisplay
	body->Redisplay();

	return TCL_OK;
}

//------------------------------------------------------------------------------------------------

static double constraints[][2] = {
	{-30.0,  30.0}, {-20.0, 20.0}, {-30.0, 30.0},	// pelvis
	{-45.0, 160.0}, {-45.0, 45.0}, {-90.0, 90.0},	// torso
	{-45.0,  90.0}, {-90.0, 90.0}, {-30.0, 30.0},	// head

	// left
	{-60.0, 180.0}, {-120.0, 120.0}, {-60.0, 180.0}, {-30.0, 30.0},	// shoulder
	{0.0, 170.0}, {-80.0, 80.0},					// elbow
	{-70.0, 80.0}, {-110.0, 110.0}, {-20.0, 30.0},	// wrist
	{-180.0, 110.0}, {-90.0, 90.0}, {-90.0, 90.0},	// hip
	{-170.0, 0.0}, 									// knee
	{-90.0, 90.0}, {-90.0, 90.0}, {-90.0, 90.0},	// ankle
	{-20.0, 60.0}, 									// foot

	// right - copied from left (change shoulderZ)
	{-60.0, 180.0}, {-120.0, 120.0}, {-180.0, 60.0}, {-30.0, 30.0},	// shoulder
	{0.0, 170.0}, {-80.0, 80.0},					// elbow
	{-70.0, 80.0}, {-110.0, 110.0}, {-20.0, 30.0},	// wrist
	{-180.0, 110.0}, {-90.0, 90.0}, {-90.0, 90.0},	// hip
	{-170.0, 0.0}, 									// knee
	{-90.0, 90.0}, {-90.0, 90.0}, {-90.0, 90.0},	// ankle
	{-20.0, 60.0}, 									// foot


	{-1e6, +1e6}, {-1e6, +1e6}, {-1e6, +1e6},		// cog translate  **NOT USED**
	{-1e6, +1e6}, {-1e6, +1e6}, {-1e6, +1e6}		// cog rotate	  **NOT USED**
};


int Body::Limits(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) const
{
int index;
Tcl_Obj *res;

    if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "joint");
        return TCL_ERROR;
    }

    if (Tcl_GetIndexFromObj(interp, objv[2], trackName, "joint", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }

	res = Tcl_NewListObj(0, NULL);
	Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj((int) constraints[index][0]));
	Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj((int) constraints[index][1]));

	Tcl_SetObjResult(interp, res);
	return TCL_OK;
}

void Body::Redisplay() const
{
	for(ListIterator<Togl> t(views); t; ++t) {
		Togl_PostRedisplay(t());
	}
}

void Body::DoOperation(pfnObj op, struct Tcl_Obj *objv[])
{
	torso.DoOperation(op, objv);
	pelvis.DoOperation(op, objv);
	neck.DoOperation(op, objv);
	headFace.DoOperation(op, objv);
	headHair.DoOperation(op, objv);

	Larm.DoOperation(op, objv);
	Rarm.DoOperation(op, objv);

	Lleg.DoOperation(op, objv);
	Rleg.DoOperation(op, objv);
}

#include "body.icc"

//------------------------------------------------------------------------------------------------

int Body::Position(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) const
{
int index, nb, jOnly, withRX, withRY, withRZ;
Tcl_Obj *res, **list;
double x,y,z;


    if (objc != 4) {
        Tcl_WrongNumArgs(interp, 2, objv, "joint {jointOnly withRX withRY withRZ}");
        return TCL_ERROR;
    }

	// get parameters
    if (Tcl_GetIndexFromObj(interp, objv[2], jointName, "joint", 0, &index) != TCL_OK) {
        return TCL_ERROR;
    }
	Tcl_ListObjGetElements(interp, objv[3], &nb, &list);
	if (nb != 4) {
		Tcl_AppendResult(interp, "bad args for filter, "
								 "should be {jointOnly withRX withRY withRZ}", (char *) NULL);
		return TCL_ERROR;
	}
	Tcl_GetBooleanFromObj(interp, list[0], &jOnly);
	Tcl_GetBooleanFromObj(interp, list[1], &withRX);
	Tcl_GetBooleanFromObj(interp, list[2], &withRY);
	Tcl_GetBooleanFromObj(interp, list[3], &withRZ);

	// do it
	Position(index, &x, &y, &z, jOnly, withRX, withRY, withRZ);

	res = Tcl_NewListObj(0, NULL);
	Tcl_ListObjAppendElement(interp, res, Tcl_NewDoubleObj(x));
	Tcl_ListObjAppendElement(interp, res, Tcl_NewDoubleObj(y));
	Tcl_ListObjAppendElement(interp, res, Tcl_NewDoubleObj(z));

	Tcl_SetObjResult(interp, res);
	return TCL_OK;
}

//------------------------------------------------------------------------------------------------

Body::Body(int nb, char *n)
{
	// variables init
	number = nb;
	name = new char[strlen(n)+1];
	sex = MAN;
	ghost = BODY;
	strcpy(name, n);
	for(int i=0; i<NB_DOF; i++) {
		joints[i] = 0.0;
	}
	views = new List<Togl>;

	// object identifiers for display lists
	torso.SetId(OBJ_TORSO); torso.SetDrawId(OBJ_TORSO_MAN);
	pelvis.SetId(OBJ_PELVIS);
	neck.SetId(OBJ_NECK); headFace.SetId(OBJ_HEADFACE); headHair.SetId(OBJ_HEADHAIR);

	Larm.arm.SetId(OBJ_LEFT_ARM); Larm.forearm.SetId(OBJ_LEFT_FOREARM);
	Larm.hand.SetId(OBJ_LEFT_HAND);
	Lleg.thigh.SetId(OBJ_LEFT_THIGH); Lleg.leg.SetId(OBJ_LEFT_LEG);
	Lleg.foot.SetId(OBJ_LEFT_FOOT); Lleg.toe.SetId(OBJ_LEFT_TOE);

	Rarm.arm.SetId(OBJ_RIGHT_ARM); Rarm.forearm.SetId(OBJ_RIGHT_FOREARM);
	Rarm.hand.SetId(OBJ_RIGHT_HAND);
	Rleg.thigh.SetId(OBJ_RIGHT_THIGH); Rleg.leg.SetId(OBJ_RIGHT_LEG);
	Rleg.foot.SetId(OBJ_RIGHT_FOOT); Rleg.toe.SetId(OBJ_RIGHT_TOE);
}

Body::~Body()
{
	delete[] name;
}

//================================================================================================


static void BodyDelete_Cmd(ClientData clientData)
{
class Body *body = (class Body *) clientData;

	// unregister the body in the views
	//for(ListIterator<Togl> t(body->views); t :++t) {
	//	t()->bodies->remove(body);
	//}

	delete body;
}


int BodyCreate_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
static int counter = 0;
int sex;
char cmd[10];
struct Body *b;

    if (argc < 2) {
        Tcl_SetResult(interp, "wrong # of arguments : body name", NULL);
        return TCL_ERROR;
    }

	b = new Body(counter, argv[1]);
	sprintf(cmd, "body%d", counter++);
	Tcl_CreateObjCommand(interp, cmd, Body_ObjCmd, 
					  (ClientData) b, (Tcl_CmdDeleteProc *) BodyDelete_Cmd);
	
	Tcl_SetResult(interp, cmd, TCL_VOLATILE);
	return TCL_OK;
}

