namespace eval body {
}

#=================================================================================================

array set body::data {
	type::color					{ all white neck #ECC face #ECC hair #762406 \
								  left:hand #ECC right:hand #ECC }
	type::wire					{ }
}

set body::builtin {}
foreach {color main arm leg} {
		Blue			#00D3F7		#20BEF7		#00A2E7
		Green			#2FC		#3B9		#7F9
		Orange			#FFB269		#FFCB51		#FFAA00
		Pink			#FFA6B6		#DF82B6		#FF8ADF
		Red				#FE3422		#D73051		#F79
		Yellow			#FFEB86		#FFDB62		#F7FB10
	} {
	set body::data(type:$color:color) [list \
			torso $main pelvis $main neck #ECC face #ECC hair #762406 \
			left:Arm $arm right:Arm $arm left:hand #ECC right:hand #ECC \
			left:Leg $leg right:Leg $leg]

	set body::data(type:$color:wire) { }

	lappend body::builtin $color
}

#-------------------------------------------------------------------------------------------------

proc body::new {name type sex} {
variable data

	set b [body $name]

	# record the type if needed
	set type [set:type $type]

	# create the body with color & wire
	eval $b setcolor $data(type:$type:color)
	$b shade all
	if {[llength $data(type:$type:wire)]} {
		eval $b wire $data(type:$type:wire)
	}
	$b set sex $sex

	return $b
}

#-------------------------------------------------------------------------------------------------

proc body::get:type {type} {
variable data
variable builtin

	if {[lsearch -exact $builtin $type] != -1} {
		return $type
	} else {
		return [list $type $data(type:$type:color) $data(type:$type:wire)]
	}
}

proc body::set:type {typedata} {
variable data

	# if type if simple (only a name) returns
	if {[llength $typedata] != 3} {
		return [lindex $typedata 0]
	}

	# create the type or update it
	set type [lindex $typedata 0]
	set data(type:$type:color) [lindex $typedata 1]
	set data(type:$type:wire) [lindex $typedata 2]
	return $type
}

proc body::delete:type {type} {
variable data

	unset data(type:$type:color)
	unset data(type:$type:wire)
}

#-------------------------------------------------------------------------------------------------

proc body::apply:type {body type} {
variable data

	eval $body setcolor $data(type:$type:color)
	if {[$body get ghost] == "body"} {
		$body shade all
		if {[llength $data(type:$type:wire)]} {
			eval $body wire $data(type:$type:wire)
		}
	}
}

proc body::list:types {} {
variable data

	set list {}
	foreach name [array name data] {
		if {[regexp {type:(.+):color} $name {} type]} {
			lappend list $type
		}
	}

	return $list 
}


#=================================================================================================

proc body::initDialog {} {
global status
variable list

	set w [toplevel .bodyConfigure]
	wm withdraw .bodyConfigure

	set f [frame .bodyConfigure.top]
	  # the check list
	  set list(w:tree) [tixTree $f.l -scrollbar auto -highlightcolor lightgray \
									-options { hlist.indicator		1
											   hlist.columns		3
											   hlist.header			true
											   hlist.width			36
											   hlist.height			8
											   hlist.indent			20 }]
		set hlist [$f.l subwidget hlist]
		# the header
		set header [tixDisplayStyle text -refwindow $hlist \
				-fg black -anchor c -padx 8 -pady 2 -font body:header]
		$hlist header create 0 -itemtype text -text Part -style $header -borderwidth 1
		$hlist header create 1 -itemtype text -text Color -style $header -borderwidth 1
		$hlist header create 2 -itemtype text -text Type -style $header -borderwidth 1
		# the entries
		set style [tixDisplayStyle text -refwindow $hlist -font body:text]
		foreach {i n} {0a "Face" 0b "Hair"
				   1 "Body" 1.1 "Torso" 1.2 "Pelvis"
				   2 "Left Arm" 2.1 "Left arm" 2.2 "Left forearm" 2.3 "Left hand"
				   3 "Right Arm" 3.1 "Right arm" 3.2 "Right forearm" 3.3 "Right hand"
				   4 "Left Leg" 4.1 "Left thigh" 4.2 "Left leg" 4.3 "Left foot"
				   5 "Right Leg" 5.1 "Right thigh" 5.2 "Right leg" 5.3 "Right foot"} {
			$hlist add $i -itemtype text -text $n -style $style
			set list($i:style) [tixDisplayStyle text -refwindow $hlist \
					-fg blue -selectforeground blue -padx 20 -font body:text]
	        $hlist item create $i 1 -itemtype text -text "Color" -style $list($i:style)
	        $hlist item create $i 2 -itemtype text -text "Shaded" -style $style
		}
		$hlist column width 0 130
		$hlist column width 2 75
		$hlist configure -command {}
		$f.l autosetmode
		$f.l close 1
		$f.l close 2
		$f.l close 3
		$f.l close 4
		$f.l close 5
		bind $hlist <Double-1> "body::action $f.l $hlist %x %y"

		# the name of the part to select/unselect
		set list(0a:part)	"head"
		set list(0b:part)	"hair"
		set list(1:part)	"body"
		set list(1.1:part)	"torso"
		set list(1.2:part)	"pelvis"
		set list(2:part)	"left:Arm"
		set list(2.1:part)	"left:arm"
		set list(2.2:part)	"left:forearm"
		set list(2.3:part)	"left:hand"
		set list(3:part)	"right:Arm"
		set list(3.1:part)	"right:arm"
		set list(3.2:part)	"right:forearm"
		set list(3.3:part)	"right:hand"
		set list(4:part)	"left:Leg"
		set list(4.1:part)	"left:thigh"
		set list(4.2:part)	"left:leg"
		set list(4.3:part)	"left:Foot"
		set list(5:part)	"right:Leg"
		set list(5.1:part)	"right:thigh"
		set list(5.2:part)	"right:leg"
		set list(5.3:part)	"right:Foot"

	  pack $f.l -side left -expand yes -fill both

	  # a small view of the body
	  set list(body) [body "__configure__"]
	  
	  set list(w:view) [togl $f.v -width 75 -height 125 -double true \
							-ident "body" -depth true -class view3D -share $status(opengl)]
	    set v $list(w:view)
		$v set view {-27.0 11.0 0.0 4.12373567275 0.0014512043551 -0.881071361601}
		$v set axis false
		$v set floor false
		$v add $list(body)
		bindtags $v "[bindtags $v] View3D"
	  pack $f.v -side left -fill both -expand true -padx 2 -pady 2

	pack $f -side top -expand yes -fill both -padx 5 -pady 5

	# the label + entry for the type name
	set f [frame $w.name]
	  label $f.l -text "Name"
	  set list(w:entry) [entry $f.e -textvariable body::name]
	  pack $f.l -side left -fill x -padx 5
	  pack $f.e -side left -fill x -padx 5 -expand true
	pack $f -side top -expand yes -fill x -pady 7

	# the row of buttons
	set f [frame $w.buttons]
	  set bo [button $f.ok -default active -text "Ok" -width 5 -underline 0 \
					-command "set __bodyconfigurestatus__ 1"]
	  set bc [button $f.cancel -text "Cancel" -width 5 -underline 0 \
					-command "set __bodyconfigurestatus__ 0"]
	  pack $f.ok $f.cancel -side left -padx 10	
	pack $f -side top -expand yes -pady 7

	# bindings
    bind $w <Alt-w> "set __bodyconfigurestatus__ 0"
    bind $w <Alt-o> "set __bodyconfigurestatus__ 1"
    bind $w <Alt-c> "set __bodyconfigurestatus__ 0"
	bind $w <Return> "set __bodyconfigurestatus__ 1"
	bind $w <Escape> "set __bodyconfigurestatus__ 0"

	# some init
	set list(v:serial) 1
}
destroy .bodyConfigure
body::initDialog

#-------------------------------------------------------------------------------------------------

proc body::action {top w x y} {
variable list

    # search the new item used for ordering
	set xmin 0
	foreach col {0 1 2} {
		if {$x <= $xmin+[$w column width $col]} {
			break;
		}
		incr xmin [$w column width $col]
	}
	set entry [$w nearest $y]

	if {$col==0} {
		# open/close the part
		if {[$list(w:tree) getmode $entry] == "open"} {
			$list(w:tree) open $entry
		} else {
			$list(w:tree) close $entry
		}
	} elseif {$col == 1} {
		# if selection of the color
		set color [lindex [$list(body) getcolor $list($entry:part)] 1]
		set color [tk_chooseColor -initialcolor $color]
		if {$color == ""} {
			return
		}
		$list(body) setcolor $list($entry:part) $color
		$w item configure $entry 1 -text "Color"
		$list($entry:style) configure -fg $color -selectforeground $color
		foreach child [$w info children $entry] {
			$list($child:style) configure -fg $color -selectforeground $color
		}
		catch {
			$w item configure [$w info parent $entry] 1 -text "----"
			$list([$w info parent $entry]:style) configure -fg black -selectforeground black
		}

	} else {
		# if selection of the shaded
		if ![$list(body) isshaded $list($entry:part)] {
			$w item configure $entry 2 -text "Wireframe"
			foreach child [$w info children $entry] {
				$w item configure $child 2 -text "Wireframe"
			}
			catch {$w item configure [$w info parent $entry] 3 -text "------"}
			$list(body) wire $list($entry:part)
		} else {
			$w item configure $entry 2 -text "Shaded"
			foreach child [$w info children $entry] {
				$w item configure $child 2 -text "Shaded"
			}
			catch {$w item configure [$w info parent $entry] 2 -text "------"}
			$list(body) shade $list($entry:part)
		}
	}
}

#-------------------------------------------------------------------------------------------------

proc body::configure {{body ""} {type ""}} {
global __bodyconfigurestatus__
variable list
variable name
variable data


	wm deiconify .bodyConfigure
	wm title .bodyConfigure "Configure type of body"
	set w [$list(w:tree) subwidget hlist]


	# configure the dialog
	if {$type == ""} {
		set name "type$list(v:serial)"
		incr list(v:serial)
	} else {
		set name $type
	}

	if {$body == ""} {
		$list(body) setcolor all white
		$list(body) shade all
		set body $list(body)
	} else {
		set color [$body getcolor all]
		eval $list(body) setcolor $color
		foreach {part value} $color {
			if {[$body isshaded $part]} {
				$list(body) wire $part
			} else {
				$list(body) shade $part
			}
		}
	}


	foreach i {0a 0b 1 1.1 1.2 1 2 2.1 2.2 2.3 3 3.1 3.2 3.3 4 4.1 4.2 4.3 5 5.1 5.2 5.3} {
		# set the color
		set colors [$body getcolor $list($i:part)]
		set color1 [lindex $colors 1]

		set same 1
		foreach {dummy color} $colors {
			if {$color != $color1} {
				set same 0
				break
			}
		}

		if {!$same} {
			$w item configure $i 1 -text "----"
			$list($i:style) configure -fg black -selectforeground black
		} else {
			$w item configure $i 1 -text "Color"
			$list($i:style) configure -fg $color1 -selectforeground $color1
		}

		# set the shade/wireframe
		set isshaded [$body isshaded $list($i:part)]
		if {$isshaded} {
			if {[string length $isshaded] > 1 && ([string first 0 $isshaded]>=0)} {
				$w item configure $i 2 -text "------"
			} else {
				$w item configure $i 2 -text "Wireframe"
			}
		} else {
			$w item configure $i 2 -text "Shaded"
		}
	}

	# wait for an answer (modal dialog)
	# give the focus
	focus $list(w:entry)
	grab set .bodyConfigure
	tkwait variable __bodyconfigurestatus__
	grab release .bodyConfigure
	wm withdraw .bodyConfigure

	if {$__bodyconfigurestatus__} {
		set color [$list(body) getcolor all]
		set data(type:$name:color) $color
		set data(type:$name:wire) {}
		foreach {part value} $color {
			if {[$list(body) isshaded $part]} {
				lappend data(type:$name:wire) $part
			}
		}
		return $name
	} else {
		return -code error "Canceled"
	}
}
