# ChkList.tcl --
#
#	   This file implements the TixCheckList widget.
#
# Copyright (c) 1996, Expert Interface Technologies
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

tixWidgetClass tixAutoCheckList {
	-classname TixAutoCheckList
	-superclass tixTree
	-method {
		getselection getstatus setstatus
	}
	-flag {
        -callback
    }
    -configspec {
        {-callback callBack CallBack ""}
	}
	-default {
		{.scrollbar					auto}
		{.doubleClick				false}
		{*Scrollbar.background		#d9d9d9}
		{*Scrollbar.relief			sunken}
		{*Scrollbar.takeFocus		0}
		{*Scrollbar.troughColor		#c3c3c3}
		{*Scrollbar.width			15}
		{*borderWidth				1}
		{*hlist.background			#c3c3c3}
		{*hlist.drawBranch			1}
		{*hlist.height				10}
		{*hlist.highlightBackground	#d9d9d9}
		{*hlist.indicator			1}
		{*hlist.indent				20}
		{*hlist.itemType			imagetext}
		{*hlist.padX				3}
		{*hlist.padY				0}
		{*hlist.relief				sunken}
		{*hlist.takeFocus			1}
		{*hlist.wideSelection		0}
		{*hlist.width				20}
	}
}

proc tixAutoCheckList:InitWidgetRec {w} {
upvar #0 $w data

	tixChainMethod $w InitWidgetRec
}

#----------------------------------------------------------------------
#
#					   Widget commands
#
#----------------------------------------------------------------------

# Helper function for getselection
#
proc tixAutoCheckList:GetSel {w var ent mode type} {
upvar #0 $w data
upvar $var img

	set ents ""
	set ok 0

	catch {
		if ![string comp [$data(w:hlist) entrycget $ent -bitmap] $img($mode)] {
			lappend ents $ent
			set ok 1
		}
	}

	if {$ok && $type=="top"} {
		return $ents
	}
	foreach child [$data(w:hlist) info children $ent] {
		set ents [concat $ents [tixAutoCheckList:GetSel $w img $child $mode $type]]
	}

	return $ents
}


# Mode can be on, off, default
#
proc tixAutoCheckList:getselection {w {type top} {mode on}} {
upvar #0 $w data

	set img(on)		 [tix getbitmap ck_on]
	set img(off)	 [tix getbitmap ck_off]
	set img(default) [tix getbitmap ck_def]

	set ents ""
	foreach child [$data(w:hlist) info children] {
		set ents [concat $ents [tixAutoCheckList:GetSel $w img $child $mode $type]]
	}
	return $ents
}

proc tixAutoCheckList:getstatus {w ent} {
upvar #0 $w data

	if {[$data(w:hlist) entrycget $ent -itemtype] == "imagetext"} {
		set img(on)	 	 [tix getbitmap ck_on]
		set img(off)	 [tix getbitmap ck_off]
		set img(default) [tix getbitmap ck_def]

		set bitmap [$data(w:hlist) entrycget $ent -bitmap]

		if {"x$bitmap" == "x$img(on)"} {
			set status on
		}
		if {"x$bitmap" == "x$img(off)"} {
			set status off
		}
		if {"x$bitmap" == "x$img(default)"} {
			set status default
		}
	}

	if {[info exists status]} {
		return $status
	} else {
		return "none"
	}
}

proc tixAutoCheckList:setstatus {w ent {mode on} {propagate 1}} {
upvar #0 $w data

	tixAutoCheckList:Select $w $ent $mode
	foreach child [$data(w:hlist) info children $ent] {
		tixAutoCheckList:setstatus $w $child $mode 0
	}

	if {$propagate} {
		tixAutoCheckList:Propagate $w $ent
	}
}

proc tixAutoCheckList:Propagate {w ent} {
upvar #0 $w data

	set parent [$data(w:hlist) info parent $ent]
	if {$parent == ""} {
		return
	}

	set bool 1
	foreach child [$data(w:hlist) info children $parent] {
		set bool [expr $bool && ![string compare [tixAutoCheckList:getstatus $w $child] on] ]
	}
	if {[expr $bool]} {
		tixAutoCheckList:Select $w $parent on
	} else {
		set bool 0
		foreach child [$data(w:hlist) info children $parent] {
			set bool [expr $bool || \
					  [string compare [tixAutoCheckList:getstatus $w $child] off] ]
		}
		if ![expr $bool] {
			tixAutoCheckList:Select $w $parent off
		} else {
			tixAutoCheckList:Select $w $parent default
		}
	}

	tixAutoCheckList:Propagate $w $parent
}


proc tixAutoCheckList:Select {w ent mode} {
upvar #0 $w data

	if {[$data(w:hlist) entrycget $ent -itemtype] == "imagetext"} {
		set img(on)	  	 ck_on
		set img(off)	 ck_off
		set img(default) ck_def

		if {[catch {
			set bitmap [tix getbitmap $img($mode)]
			$data(w:hlist) entryconfig $ent -bitmap $bitmap
		}]} {
			# must be the "none" mode
			#
			catch {
				$data(w:hlist) entryconfig $ent -bitmap ""
			}
		}

	    if {$data(-callback) != ""} {
			tixEvalCmdBinding $w $data(-callback) "" $ent $mode
		}
	}

	return $mode
}

proc tixAutoCheckList:HandleCheck {w ent} {
	upvar #0 $w data

	if {[$data(w:hlist) entrycget $ent -itemtype] == "imagetext"} {
		set img(on)	  [tix getbitmap ck_on]
		set img(off)	 [tix getbitmap ck_off]
		set img(default) [tix getbitmap ck_def]

		set curMode [tixAutoCheckList:getstatus $w $ent]

		case $curMode {
			on {
				tixAutoCheckList:setstatus $w $ent off
			}
			off {
				tixAutoCheckList:setstatus $w $ent on
			}
			none {
				return
			}
			default {
				tixAutoCheckList:setstatus $w $ent on
			}
		}
	}
}

proc tixAutoCheckList:Command {w B} {
upvar #0 $w data
upvar $B bind

	set ent [tixEvent flag V]
	tixAutoCheckList:HandleCheck $w $ent

#	tixChainMethod $w Command $B
}

proc tixAutoCheckList:BrowseCmd {w B} {
upvar #0 $w data
upvar $B bind

	set ent [tixEvent flag V]

	case [tixEvent type] {
		{<ButtonPress-1> <space>} {
			tixAutoCheckList:HandleCheck $w $ent
		}
	}

#	tixChainMethod $w BrowseCmd $B 
}
