namespace eval figEditor {
	variable list {}

	image create pixmap tbf_x -file pixmap/tb_fig_x.xpm
	image create pixmap tbf_y -file pixmap/tb_fig_y.xpm
	image create pixmap tbf_z -file pixmap/tb_fig_z.xpm
	image create pixmap tbf_ik -file pixmap/tb_fig_ik.xpm
	image create pixmap tbf_axe -file pixmap/tb_fig_axe.xpm
}

#-------------------------------------------------------------------------------------------------

proc figEditor::projection {w wdata az el axe} {
global status
variable $wdata
upvar 0 $wdata data
    
	# the windows
    togl $w -width 60 -height 60 -double true -depth true \
		    -class view3D -share $status(opengl) -ident "$wdata $axe"
    $w set elevation $el
    $w set azimut $az
    if {$axe != "Y"} {$w set ytrans -1}
    $w set axis false
    $w ortho
    $w set zoom .87
	replaceBindtags $w "OpenGL" "View3DProjection"
    pack $w -side left -expand yes -fill both

	# popup menu
    menu $w.m -tearoff 0 -bd 1
      $w.m add check -label "Flip axe" -command "view3D::flip $w $az $el" \
		  -font popupmenu -variable figEditor::${wdata}(dir:$axe) -onvalue -1 -offvalue 1
      $w.m add check -label "Force wireframe" -variable figEditor::${wdata}(forcewire:$axe) \
		  -font popupmenu -command "$w set forcewire \[set figEditor::${wdata}(forcewire:$axe)\]"

	# some bindings
    bind $w <ButtonPress-3> "tk_popup $w.m %X %Y"

# 	bind $w <ButtonPress-2> "view3D::pickProj:setup ::figEditor::$data(w:top) %W %x %y $axe"
# 	bind $w <ButtonPress-2> "+figEditor::joint:display ::figEditor::$data(w:top) %W %x %y"
# 	bind $w <Button2-Motion> "view3D::pickProj ::figEditor::$data(w:top) %W %x %y $axe"
# 	bind $w <Button2-Motion> "+figEditor::joint:display ::figEditor::$data(w:top)"
	bind $w <ButtonPress-1> \
		"if {\[view3D::pickProj:setup ::figEditor::$data(w:top) %W %x %y $axe\]} {break}"
	bind $w <ButtonPress-1> "+figEditor::joint:display ::figEditor::$data(w:top) %W %x %y"
	bind $w <Button1-Motion> \
		"if {\[view3D::pickProj ::figEditor::$data(w:top) %W %x %y $axe\]} {break}"
	bind $w <Button1-Motion> "+figEditor::joint:display ::figEditor::$data(w:top)"
	return $w
}

#-------------------------------------------------------------------------------------------------

proc figEditor::views {w body} {
global status
variable $w
upvar 0 $w data

	# 3D view
	set ww [$w.top add 3d -min 70 -size 275 -expand 1]
	  set data(w:3d) [togl $ww.t -width 300 -height 400 -ident "$w 3d" \
			  -double true -depth true -class view3D -share $status(opengl)]
		$data(w:3d) set elevation 22
		$data(w:3d) set azimut -35
		$data(w:3d) set ytrans -0.88
	 	$data(w:3d) set zoom 4
		$data(w:3d) set localaxis:color1 yellow
		$data(w:3d) set localaxis:color2 red
		replaceBindtags $data(w:3d) "OpenGL" "View3D"
		pack $ww.t -side top -fill both -expand true -padx 2 -pady 2

	# bindings
	set top $data(w:top)
# 	bind $data(w:3d) <ButtonPress-2> "figEditor::pick3D:setup $top %W %x %y sphere"
# 	bind $data(w:3d) <Button2-Motion> "figEditor::pick3D:xz $top %W %x %y"
# 	bind $data(w:3d) <ButtonRelease-2> "$data(w:3d) set spheres {false false false}"

# 	bind $data(w:3d) <Control-ButtonPress-2> "figEditor::pick3D:setup $top %W %x %y sphere no"
# 	bind $data(w:3d) <Control-Button2-Motion> "figEditor::pick3D:xz $top %W %x %y"
# 	bind $data(w:3d) <Control-ButtonRelease-2> "$data(w:3d) set spheres {false false false}"

# 	bind $data(w:3d) <Shift-ButtonPress-2> "figEditor::pick3D:setup $top %W %x %y disk"
# 	bind $data(w:3d) <Shift-Button2-Motion> "figEditor::pick3D:y $top %W %x %y"
# 	bind $data(w:3d) <Shift-ButtonRelease-2> "$data(w:3d) set disk false"

# 	bind $data(w:3d) <ButtonPress-2> "+figEditor::joint:display $top %W %x %y"
# 	bind $data(w:3d) <Button2-Motion> "+figEditor::joint:display $top"
# 	bind $data(w:3d) <Shift-ButtonPress-2> "+figEditor::joint:display $top %W %x %y"
# 	bind $data(w:3d) <Shift-Button2-Motion> "+figEditor::joint:display $top"

	bind $data(w:3d) <ButtonPress-1> \
		"if {\[figEditor::pick3D:setup $top %W %x %y sphere\]} {
			figEditor::joint:display $top %W %x %y
			break
		}"
	bind $data(w:3d) <Button1-Motion> \
		"if {\[figEditor::pick3D:xz $top %W %x %y\]} {
			figEditor::joint:display $top
			break
		}"
	bind $data(w:3d) <ButtonRelease-1> "$data(w:3d) set spheres {false false false}"

	bind $data(w:3d) <Control-ButtonPress-1> \
		"if {\[figEditor::pick3D:setup $top %W %x %y sphere no\]} {
			figEditor::joint:display $top %W %x %y
			break
		}"
	bind $data(w:3d) <Control-Button1-Motion> \
		"if {\[figEditor::pick3D:xz $top %W %x %y\]} {
			figEditor::joint:display $top
			break
		}"
	bind $data(w:3d) <Control-ButtonRelease-1> "$data(w:3d) set spheres {false false false}"

	bind $data(w:3d) <Shift-ButtonPress-1> \
		"if {\[figEditor::pick3D:setup $top %W %x %y disk\]} {
			figEditor::joint:display $top %W %x %y
			break
		}"
	bind $data(w:3d) <Shift-Button1-Motion> \
		"if {\[figEditor::pick3D:y $top %W %x %y\]} {
			figEditor::joint:display $top
			break
		}"
	bind $data(w:3d) <Shift-ButtonRelease-1> "$data(w:3d) set disk false"


	bind $data(w:3d) <ButtonPress-3> "figEditor::actionMenu $w $body %x %y %X %Y"

	# Projections
	set ww [$w.top add proj -min 70 -size 125 -expand 1]
	  # XY
	  set x [frame $ww.x]
	  set data(w:XY) [projection $x.p $w 0 0 Z]
	  set data(dir:Z) 1
	  # YZ
	  set y [frame $ww.y]
	  set data(w:YZ) [projection $y.p $w 90 0 X]
	  set data(dir:X) 1
	  # XZ
	  set z [frame $ww.z]
	  set data(w:XZ) [projection $z.p $w 0 90 Y]
	  set data(dir:Y) 1
	
	  pack $x $y $z -side top -expand true -fill both -padx 2 -pady 2

	# add the body to edit
	set data(v:body) $body
	$data(w:3d) add $body
	$data(w:XY) add $body
	$data(w:YZ) add $body
	$data(w:XZ) add $body

	# center body	
	$data(w:3d) set jointsonly true
	$data(w:XY) set jointsonly true
	$data(w:YZ) set jointsonly true
	$data(w:XZ) set jointsonly true
}

#-------------------------------------------------------------------------------------------------

proc figEditor::new {w routineEditor body {pos ""} {size ""}} {
global status
variable $w
upvar 0 $w data
variable list

	if {[winfo exists $w]} {
		raise $w
		return
	}

	# register the editor
	lappend list $w

	# create the window
	set data(w:top) $w
	toplevel $w
	if {$pos != "" || $size != ""} {
		wm geometry $w $size$pos
	}
	if {$routineEditor == ""} {
		wm title $w "Pose Editor for Library"
	} else {
		wm title $w "Pose Editor <[$body get name]>"
	}
	wm protocol $w WM_DELETE_WINDOW "figEditor::close $w"
	bind $w <Alt-w> "after idle \"figEditor::close $w\""
	bind $w <Escape> "after idle \"figEditor::close $w\""

	# init
	set data(v:routineEditor) $routineEditor

	# the status bar
	set s [frame $w.s]
	label $s.help -relief sunken -bd 1 -font status -width 40 \
			-textvariable figEditor::${w}(st:help)
	pack $s.help -expand yes -fill x -side left

	# the views
    tixPanedWindow $w.top -paneborderwidth 0 -separatorbg gray50 -orient horizontal
	views $w $body

	# the controls to display joint+values
	menubutton $w.joint -indicatoron 1 -menu $w.joint.menu -width 10 \
			-textvariable figEditor::${w}(v:joint) -relief raised \
			-bd 1 -padx 3 -pady 3 -anchor w -font menubutton:small -direction above
	set m [menu $w.joint.menu -tearoff 0 -font menubutton:small \
			-activeborderwidth 1 -bd 1]
	foreach {j short} {
				torso "t" pelvis "p" head "h" --- ""
				left:shoulder "ls" left:elbow "le" left:wrist "lw" --- ""
				right:shoulder "rs" right:elbow "re" right:wrist "rw" --- ""
				left:hip "lh" left:knee "lk" left:ankle "la" left:foot "lf" --- ""
				right:hip "rh" right:knee "rk" right:ankle "ra" right:foot "rf" } {
		if {$j == "---"} {
			$m add separator
		} else {
			$m add command -label $j -command "figEditor::joint:display $w direct $j"
			bind $w $short "figEditor::joint:display $w direct $j"
		}
	}
	label $w.sep -width 1
	_entry $w.x $w "X" X
	_entry $w.y $w "Y" Y
	_entry $w.z $w "Z" Z
	_entry $w.ty $w "TY" TY
	joint:display $w direct torso

	# toolbar + button
	set f [frame $w.but]
	toolbar::new $f.tb
	toolbar::add:check $f.tb rx -bd 1 -image tbf_x -tip "Allow rotation around X" \
			-oncommand "figEditor::filter $w X true" -offcommand "figEditor::filter $w X false"
	toolbar::add:check $f.tb ry -bd 1 -image tbf_y -tip "Allow rotation around Y" \
			-oncommand "figEditor::filter $w Y true" -offcommand "figEditor::filter $w Y false"
	toolbar::add:check $f.tb rz -bd 1 -image tbf_z -tip "Allow rotation around Z" \
			-oncommand "figEditor::filter $w Z true" -offcommand "figEditor::filter $w Z false"
	toolbar::add:separator $f.tb 10
	toolbar::add:radio $f.tb ik -bd 1 -image tbf_ik -tip "Intelligent mode" \
			-group 1 -variable figEditor::${w}(v:ik) -state disabled
	toolbar::add:radio $f.tb axe -bd 1 -image tbf_axe -tip "Simple rotation around axes mode" \
			-group 1 -variable figEditor::${w}(v:ik)
	pack $f.tb -side left -padx 5
	set figEditor::${w}(v:ik) axe

	if {$routineEditor == ""} {
		button $f.sync -text "Update library" -command "figEditor::library $w"
	} else {
		button $f.sync -text "Sync" -command "routine::update $routineEditor joints $body"
	}
	button $f.dismiss -text "Dismiss" -command "figEditor::close $w"
	pack $f.sync $f.dismiss -fill x -expand true -padx 2 -pady 2 -side left


	# the layout
	grid $w.top   -		 -    -     -    -     -    -     -		-		-sticky news
	grid $w.joint $w.sep $w.x $w.x2 $w.y $w.y2 $w.z $w.z2 $w.ty $w.ty2	-sticky news
	grid $w.but   -		 -    -     -    -     -    -     -    	-		-sticky news
	grid $w.s     -		 -    -     -    -     -    -     -     -	-sticky news -padx 1 -pady 1
	grid rowconfig    $w {0} -weight 1
	grid rowconfig    $w {1 2} -weight 0
	grid columnconfig $w {0 1 2 4 6 8} -weight 0
	grid columnconfig $w {3 5 7 9} -weight 1

	# some bindings for body manipulation
    bind $w <KeyPress-x> "set figEditor::${w}(v:mode) X; $w.x configure -bg darkseagreen2"
    bind $w <KeyRelease-x> "set figEditor::${w}(v:mode) {}; $w.x configure -bg lightgray"
    bind $w <KeyPress-z> "set figEditor::${w}(v:mode) Z; $w.z configure -bg darkseagreen2"
    bind $w <KeyRelease-z> "set figEditor::${w}(v:mode) {}; $w.z configure -bg lightgray"
    bind $w <KeyPress-Shift_L> "+$w.y configure -bg darkseagreen2"
    bind $w <KeyRelease-Shift_L> "+$w.y configure -bg lightgray"
    bind $w <KeyPress-Shift_R> "+$w.y configure -bg darkseagreen2"
    bind $w <KeyRelease-Shift_R> "+$w.y configure -bg lightgray"

	if {$data(v:routineEditor) != ""} {
		bindtags $w.top "[bindtags $w.top] KeyRoutine"
	}

	# the drag & drop facility
	drag&drop::target add $w -type "pose" -cursor target -cmd "figEditor::drop:cmd"

	# some init
	helpCreate $w.top figEditor::${w}(st:help) figEditor::help
	setTime $w
	set data(v:mode) ""

	toolbar::check:invoke $f.tb rx
	toolbar::check:invoke $f.tb ry;	toolbar::check:invoke $f.tb ry		;# set off
	toolbar::check:invoke $f.tb rz
}

proc figEditor::_entry {w wdata text var} {
global status

	label $w -font entry:small -text $text
	entry ${w}2 -font entry:small -textvariable figEditor::${wdata}(v:$var) \
			-relief sunken -bd 1 -justify right -width 7
	bind ${w}2 <FocusOut> "figEditor::validateEntry $wdata $var"
	bind ${w}2 <Return> "figEditor::validateEntry $wdata $var"
}

proc figEditor::validateEntry {w var} {
variable $w
upvar 0 $w data

	set val $data(v:$var)
	if {[scan $val "%g%s" {} {}] != 1} {
		set data(v:$var) 0
		set val 0
	}

	$data(v:body) set $data(v:joint):$var $val
	set data(v:$var) [format "%.2f" $val]
}

#=================================================================================================
    
proc figEditor::close {w} {
variable $w
upvar 0 $w data
variable list

	set i [lsearch -exact $list $w]
	set list [lreplace $list $i $i]

	if {$data(v:routineEditor) == ""} {
		library::callback:update cancel
	}

	destroy $w
	# remove array associated with window
	catch {unset data}
}

proc figEditor::setTime {w} {
variable listRestore
variable $w
upvar 0 $w data

	# set the restore values for joints
	foreach part {torso pelvis head
				  left:shoulder left:elbow left:wrist
				  right:shoulder right:elbow right:wrist
				  left:hip left:knee:X left:ankle left:foot:X
				  right:hip right:knee:X right:ankle right:foot:X } {
		set listRestore($w:$part) [$data(v:body) get $part]
	}
	# update the values displayed for the joint
	joint:display $w
}

#-------------------------------------------------------------------------------------------------

proc figEditor::joint:check {w axe} {
variable $w
upvar 0 $w data

	if {[catch {$data(v:body) get $data(v:joint):$axe}]} {
		set data(t:j$axe) 0
		set data(v:$axe) 0.00
		$data(w:top).[string tolower $axe]2 configure -state disabled -fg gray
	} else {
		set data(t:j$axe) -1
		$data(w:top).[string tolower $axe]2 configure -state normal -fg black
	}
}


proc figEditor::joint:display {w {view {}} {x 0} {y 0}} {
variable $w
upvar 0 $w data

	# if new joint to display
	if {$view != ""} {
		if {$view != "direct"} {
			set what [$view pickjoint $x $y]
		    if {$what != ""} {
				set data(v:joint) [lindex $what 1]
			} else {
				return
			}
		} else {
			set data(v:joint) $x
		}
		joint:check $w X
		joint:check $w Y
		joint:check $w Z
		joint:check $w TY
	}

	# display the values
	if {$data(t:jX)} { set data(v:X) [format "%.2f" [$data(v:body) get $data(v:joint):X]] }
	if {$data(t:jY)} { set data(v:Y) [format "%.2f" [$data(v:body) get $data(v:joint):Y]] }
	if {$data(t:jZ)} { set data(v:Z) [format "%.2f" [$data(v:body) get $data(v:joint):Z]] }
	if {$data(t:jTY)} { set data(v:TY) [format "%.2f" [$data(v:body) get $data(v:joint):TY]] }
}

#=================================================================================================

proc figEditor::filter {w axe mode} {
variable $w
upvar 0 $w data

	$data(w:3d) set withR$axe $mode
	$data(w:XY) set withR$axe $mode
	$data(w:YZ) set withR$axe $mode
	$data(w:XZ) set withR$axe $mode
}

#-------------------------------------------------------------------------------------------------

proc figEditor::pick3D:setup {w wview x y type {reset yes}} {
variable $w
upvar 0 $w data


	set what [$wview pickjoint $x $y]
#puts "pick3D:setup in $wview  --  $what"

    if {$what != ""} {
		set data(t:body) [lindex $what 0]
		set data(t:middle) [lindex $what 1]
		if {$data(v:ik) == "ik" && [regexp {.*:(elbow|knee)} $data(t:middle)]} {
			# IK mode
			switch -exact $data(t:middle) {
				left:elbow	{ set data(t:end) 	 left:wrist 
							  set data(t:origin) left:shoulder }
				right:elbow	{ set data(t:end)	 right:wrist 
							  set data(t:origin) right:shoulder }
				left:knee	{ set data(t:end)	 left:ankle 
							  set data(t:origin) left:hip }
				right:knee	{ set data(t:end)	 right:ankle 
							  set data(t:origin) right:hip }
			}
puts "IK setup --> $data(t:end)"

			set pos [$data(t:body) position $data(t:origin) [$wview get filters]]
puts "    $data(t:origin) = $pos"
			set data(t:origin:x) [lindex $pos 0]
			set data(t:origin:y) [lindex $pos 1]
			set data(t:origin:z) [lindex $pos 2]

			set pos [$data(t:body) position $data(t:end) [$wview get filters]]
puts "    $data(t:end) = $pos"

			set data(t:sx) $x
			set data(t:sy) $y

			set data(t:realik) 1
		} else {

			# normal mode
			view3D::pick3D:setup $w $wview $x $y $type $data(v:mode) $reset
			set data(t:realik) 0
		}
		set data(t:ok) 1
	} else {
		set data(t:ok) 0
	}
	return $data(t:ok)
}

#-------------------------------------------------------------------------------------------------

proc figEditor::pick3D:xz {w wview x y} {
variable $w
upvar 0 $w data

    if {$data(t:ok)} {
		if {$data(t:realik)} {
puts "IK motion xz"

			#set x [expr ($data(t:sx)-$x)*.05-$data(t:origin:x)]
			#set y 0
			#set z [expr ($data(t:sy)-$y)*.05-$data(t:origin:z)]
			set x [expr ($data(t:sx)-$x)*.5]
			set y 0
			set z [expr ($data(t:sy)-$y)*5]
			set R [expr sqrt($x*$x+$y*$y+$z*$z) * 0.64]		;# 100 cm  vs  0.64
			set khi [expr atan($z/$x)*180.0/3.141592654]
			set phi [expr atan($y/sqrt($x*$x+$z*$z))*180.0/3.141592654]

			set theta [expr  -4.0 + 1.10*$R + 0.90*$phi ]
			set beta  [expr  39.4 + 0.54*$R - 1.06*$phi ]
			set eta   [expr  13.2 + 0.86*$R + 0.11*$phi ]
			set alpha [expr -10.0 + 1.08*$R - 0.35*$phi ]

puts "($x $y $z) with $R = $theta  $beta  $eta  $alpha"
			$data(t:body) set $data(t:origin):X $eta
			$data(t:body) set $data(t:origin):Z $theta
			$data(t:body) set $data(t:middle):X $beta
			$data(t:body) set $data(t:middle):Y $alpha

		} else {
			view3D::pick3D:xz $w $x $y sphere
		}
	}
	return $data(t:ok)
}

#proc toto {x y z} {
#
#	set R [expr sqrt($x*$x+$y*$y+$z*$z)]
#	set khi [expr atan2($z,$x)*180.0/3.141592654]
#	set phi [expr atan2($y,sqrt($x*$x+$z*$z))*180.0/3.141592654]
#
#	set theta [expr  -4.0 + 1.10*$R + 0.90*$phi ]
#	set beta  [expr  39.4 + 0.54*$R - 1.06*$phi ]
#	set eta   [expr  13.2 + 0.86*$R + 0.11*$phi ]
#	set alpha [expr -10.0 + 1.08*$R - 0.35*$phi ]
#
#puts "($x $y $z) with ($R $khi $phi) = $theta  $beta  $eta  $alpha"
#}


#-------------------------------------------------------------------------------------------------

proc figEditor::pick3D:y {w wview x y} {
variable $w
upvar 0 $w data

    if {$data(t:ok)} {
		if {$data(t:realik)} {
puts "IK motion y"
		} else {
			view3D::pick3D:y $w $x $y disk
		}
	}
	return $data(t:ok)
}


#=================================================================================================

proc figEditor::drop:cmd {w type sent} {
variable $w
upvar 0 $w data

	foreach entry $sent {
		$data(v:body) set [lindex $entry 0] [lindex $entry 1]
	}
}

#-------------------------------------------------------------------------------------------------

proc figEditor::library {w} {
variable $w
upvar 0 $w data


	set view [$data(w:3d) get all]
	figEditor::close $w
	library::callback:update $view
}


#=================================================================================================


set m [menu .__figeditBody__ -tearoff 0 -font popupmenu -bd 1 -activeborderwidth 1]
  $m add command
  $m add command
  $m add separator
  $m add command
  $m add separator
  $m add cascade -label "Add pose to library" -menu $m.lib
	menu $m.lib -tearoff 0 -font popupmenu -bd 1 -activeborderwidth 1
	  $m.lib add command -label "Arm"
	  $m.lib add command -label "Body"
	  $m.lib add command -label "Foot"
	  $m.lib add command -label "Leg"
	  $m.lib add command -label "Pose"
  $m add separator
  $m add check -label "Force wireframe" -variable __figeditCheck__


array set figEditor::listReset {
	torso {0 0 0} pelvis {0 0 0} head {0 0 0}
	left:shoulder {0 0 0 0} left:elbow {0 0} left:wrist {0 0 0}
	right:shoulder {0 0 0 0} right:elbow {0 0} right:wrist {0 0 0}
	left:hip {0 0 0} left:knee:X 0 left:ankle {0 0 0} left:foot:X 0
	right:hip {0 0 0} right:knee:X 0 right:ankle {0 0 0} right:foot:X 0 }


proc figEditor::actionMenu {w body x y X Y} {
variable listReset
variable listRestore
variable $w
upvar 0 $w data

	set m .__figeditBody__

    set what [$data(w:3d) pickjoint $x $y]
    if {$what != ""} {
		set body [lindex $what 0]
		set part [lindex $what 1]
		set result [array get listReset $part*]
		$m entryconfigure 0 -state normal -label "Reset $part" \
				-command "$body set $result; figEditor::joint:display $w direct $part"
		set result [array get listRestore [winfo toplevel $w]:$part*]
		$m entryconfigure 1 -state normal -label "Restore $part" \
				-command "$body set $part {[lindex $result 1]}; \
						  figEditor::joint:display $w direct $part"
		$m entryconfigure 3 -label "View track of $part ..." -state normal \
				-command "viewTrack::new .t_$body $data(v:routineEditor) $body $part"

    } else {
		$m entryconfigure 0 -state disabled -label "Reset part"
		$m entryconfigure 1 -state disabled -label "Restore part"
		$m entryconfigure 3 -state disabled -label "View track ..."
	}

	if {$data(v:routineEditor) == ""} {
		$m entryconfigure 5 -state disabled
	} else {
		$m entryconfigure 5 -state normal
		$m.lib entryconfigure 0 -command "library::pose:dialog add $data(w:3d) $body $m.lib"
		$m.lib entryconfigure 1 -command "library::pose:dialog add $data(w:3d) $body $m.lib"
		$m.lib entryconfigure 2 -command "library::pose:dialog add $data(w:3d) $body $m.lib"
		$m.lib entryconfigure 3 -command "library::pose:dialog add $data(w:3d) $body $m.lib"
		$m.lib entryconfigure 4 -command "library::pose:dialog add $data(w:3d) $body $m.lib"
	}

	set toggle [expr ![$data(w:3d) get forcewire]]
	global __figeditCheck__
	set __figeditCheck__ [$data(w:3d) get forcewire]
	$m entryconfigure 7 -command "$data(w:3d) set forcewire $toggle"


	tk_popup $m $X $Y
}

#=================================================================================================

# array set figEditor::help {
# 	normal	"<1> Rotate view    <2> Rotate joint (<x> <z>)   <3> Action menu"
# 	shift	"<1> Zoom view    <2> Twist joint     <3> Action menu"
# 	control	"<1> Translate view    <2> Rotate joint no <y> reset (<x> <z>)    <3> Action menu"
# 	alt		"<1> Transalte center    <2> Rotate joint (<x> <z>)   <3> Action menu"
# 	off		""}
array set figEditor::help {
	normal	"<L> Rotate view    <L-body> Rotate joint (<x> <z>)   <R> Action menu"
	shift	"<L> Zoom view    <L-body> Twist joint     <R> Action menu"
	control	"<L> Translate view    <L-body> Rotate joint no <y> reset (<x> <z>)    <R> Action menu"
	alt		"<L> Transalte center    <L-body> Rotate joint (<x> <z>)   <R> Action menu"
	off		""}
