#=================================================================================================
#
#	Management of the routine tab
#

proc library::routine:category:setlist:onedir {dir selection parent} {
variable data


	set dirs [lsort -dictionary [glob -nocomplain $dir/*]]
	set i 1
	foreach f $dirs {
		if {[file isdirectory $f]} {
			if {$parent != ""} {
				set path [string range $parent 0 [expr [string length $parent]-2]]
				$data(w:routine:class) entryconfigure $path -style $data(v:style:left1b)
			}
			$data(w:routine:class) add $parent$i -text [file tail $f] -data "$f/*" \
					-style $data(v:style:left1)
			# do inner level
			routine:category:setlist:onedir $f $selection $parent$i.

			if {![string compare $selection $f]} {
				set data(t:selected) $parent$i
			}
			incr i
		}
	}
}

proc library::routine:category:setlist {{selection {}}} {
global status
variable data


	$data(w:routine:class) delete all
	$data(w:routine:class) add 0 -text "< Root >" -data "$status(dir:routine)/$data(v:class)/*" \
			-style $data(v:style:left1)

	set data(t:selected) 0
	# populate the tree
	routine:category:setlist:onedir "$status(dir:routine)/$data(v:class)" $selection ""
	$data(w:routine:treeclass) autosetmode
#foreach c [$data(w:routine:class) conf] {puts $c}
	# load selected / default category
	routine:category:load $data(t:selected)
	$data(w:routine:class) selection set $data(t:selected)
	$data(w:routine:class) anchor set $data(t:selected)

variable status
	set status(comment) ""
	set status(modif) ""
}

proc library::routine:category:load {entry} {
global status
variable data

	# read routines
	set idx 0
	set r [::routine]
	set list [glob -nocomplain [$data(w:routine:class) info data $entry]]
	set data(v:routine:class) $entry

	set rtns ""
	foreach f $list {
		if {[file isfile $f]} {
			# read the data
			$r parse $f
			set f [string range $f [expr [string length $status(dir:routine)]+1] end]
			lappend rtns [list $f [$r get name] [format "%g" [expr [$r get beats]/1000.0]] \
						  [$r get dancers] [$r get description]]
		}
	}
	rename $r {}			;# delete the routine


	set exclude [string range $main::data(v:filename) \
						[expr [string length $status(dir:routine)]+1] end]

    $data(w:routine) delete all
	foreach r [lsort -dictionary -index 1 $rtns] {
		if {$exclude == [lindex $r 0]  ||  [routine:iscircular $exclude [lindex $r 0]]} {
			set state disabled
		} else {
			set state normal
		}

		set row [$data(w:routine) add $idx -text [lindex $r 1] \
				-style $data(v:style:$state:left) -data [lindex $r 0] -state $state ]
		$data(w:routine) item create $row 1 -itemtype text -text [lindex $r 2] \
				-style $data(v:style:$state:right)
	    $data(w:routine) item create $row 2 -itemtype text -text [lindex $r 3] \
				-style $data(v:style:$state:center)
	    $data(w:routine) item create $row 3 -itemtype text -text [lindex $r 4] \
				-style $data(v:style:$state:left)
		incr idx
	}
}

proc library::routine:reload {} {
variable data

	set entry [$data(w:routine:class) info selection]
	if {$entry != ""} {
		routine:category:load $entry
	}
}

#-------------------------------------------------------------------------------------------------

proc library::routine:handle {w y} {
global status
	set path $status(dir:routine)

variable status

	focus $w
	$w selection clear
	$w anchor clear
	set entry [$w nearest $y]
	if {$entry != ""} {
		if {[$w entrycget $entry -state] == "disabled"} {
			return
		}
        $w selection set $entry
        $w anchor set $entry
		set status(comment) [$w info data $entry]
		set status(modif) [clock format [file mtime $path/[$w info data $entry]] \
								-format "%d %b %Y, %H:%M"]
	}
}

#-------------------------------------------------------------------------------------------------

proc library::routine:category:actionMenu {w x y X Y} {

	set entry [$w nearest $y]
	if {$entry == ""} {
		return
	}
	$w anchor set $entry

	if {$entry == "0"} {
		$w.m entryconfigure 1 -state disabled
	} else {
		$w.m entryconfigure 1 -state normal \
				-command "library::routine:category:delete $w [list $entry]"
	}
	$w.m entryconfigure 0 -command "library::routine:category:new $w [list $entry]"

	tk_popup $w.m $X $Y
}

proc library::routine:category:new {w entry} {
variable data

	if {[getStringFromDialog "Library" "Enter the name for the new sub-category" Name "" name]} {
		# check the name
		if {$name == ""} {
			return
		}
		if {![string match {[-a-zA-Z0-9_. ]*} $name]} {
			tk_messageBox -icon "error" -type ok -title "Error" \
					-message "Name should only contains letters, digits, space, dot, - and _"
			return
		}
		set root [file dirname [$w entrycget $entry -data]]
		if {[file exists $root/$name]} {
			tk_messageBox -icon "error" -type ok -title "Error" \
					-message "The name '$name' is already used."
			return
		}
		# generate new filename & update list
		file mkdir $root/$name
		routine:category:setlist $root/$name
	}
}

proc library::routine:category:delete {w entry} {
global status
variable data


	# get the category
	set category [string range [$w entrycget $entry -data] \
						[expr [string length $status(dir:routine)]+1] end]
	set category [file dirname $category]
	# check if used somewhere else
	routine:check:category $category

	if {[llength $data(t:list)] > 0} {
		routine:check:showconflict "categories"

	} else {
		if {[tk_messageBox -icon "question" -type yesno -default yes -title "Error" \
						-message "Do you really want to delete this category ?"] == "no"} {
			return
		}

		# do it
		file delete -force -- [file dirname [$w entrycget $entry -data]]
		set parent [$w info parent $entry]
		$w delete entry $entry
		if {[$w info children $parent] == {}} {
			$w entryconfigure $parent -style $data(v:style:left1)
		}
	}
}

#-------------------------------------------------------------------------------------------------

proc library::routine:doubleClick {w x y} {
global status

	set entry [$w nearest $y]
	if {$entry == "" || [$w entrycget $entry -state]=="disabled"} {
		return
	}
	main::open load [list $status(dir:routine)/[$w entrycget $entry -data]]
}

proc library::routine:actionMenu {w x y X Y} {
global status

	set entry [$w nearest $y]
	if {$entry == "" || [$w entrycget $entry -state]=="disabled"} {
		return
	}
	$w anchor set $entry

	$w.m entryconfigure 0 -command "library::routine:info $w [list $entry]"
	$w.m entryconfigure 1 \
			-command "main::open load [list $status(dir:routine)/[$w entrycget $entry -data]]"
	$w.m entryconfigure 3 -command "library::routine:delete $w [list $entry]"

	tk_popup $w.m $X $Y
}

proc library::routine:delete {w entry} {
global status
variable data

	# check if used somewhere else
	routine:check:figure [$w entrycget $entry -data]
	if {[llength $data(t:list)] > 0} {
		routine:check:showconflict "routines"

	} else {
		if {[tk_messageBox -icon "question" -type yesno -default yes -title "Error" \
						-message "Do you really want to delete this routine ?"] == "no"} {
			return
		}
		# do it
		file delete -force -- $status(dir:routine)/[$w entrycget $entry -data]
		$w delete entry $entry
	}
}

proc library::routine:info {w entry} {
global status
variable data


	routine:check:figure [$w entrycget $entry -data]
	set r [::routine] 
	$r parse [$w entrycget $entry -data]

	# create the window
	set top [buildModalDialog "Library - Info on '[$w entrycget $entry -data]'"]
	tixLabelFrame $top.i -label "Information"
    set info [$top.i subwidget frame]
	label $info.l_0 -text "Name :" -font library:class
	label $info.0 -text [$r get name] -font library:class -justify left \
			-bd 1 -bg gray92 -relief sunken
	label $info.l_1 -text "Description :" -font library:class
	label $info.1 -text [$r get description] -font library:class -justify left \
			-bd 1 -bg gray92 -relief sunken
	label $info.l_2 -text "Copyright :" -font library:class
	label $info.2 -text [$r get copyright] -font library:class -justify left \
			-bd 1 -bg gray92 -relief sunken
	label $info.l_3 -text "Beats :" -font library:class
	label $info.3 -text "[expr [$r get beats]/1000] (in [$r get bpm] bpm)" \
			-font library:class -bd 1 -bg gray92 -relief sunken
	label $info.l_5 -text "Movies :" -font library:class
	frame $info.5 -bd 1 -relief sunken -bg gray92
	set i 0
	set movies [$r get movies]
	if {[llength $movies] != 0} {
		foreach m $movies {
			label $info.5.$i -text $m -font library:class -bg gray92
			pack $info.5.$i -expand true -fill x
			incr i
		}
	} else {
		label $info.5.$i -text "No movies" -font library:class -bg gray92 -fg gray
		pack $info.5.$i -expand true -fill x
	}
	label $info.l_6 -text "Dancers :" -font library:class
	frame $info.6 -bd 1 -relief sunken
	for {set i 0} {$i<[$r get dancers]} {incr i} {
		label $info.6.$i -text "[$r get dancername $i] ([$r get dancersex $i])" \
			-font library:class -bg gray92
		pack $info.6.$i -expand true -fill x
	}
	rename $r {}			;# delete the routine
	# used by
	tixLabelFrame $top.u -label "Used by"
    set use [$top.u subwidget frame]
	label $use.l_7 -text "Figure used by routines" -font library:class -padx 3
	routine:list:usedby $use.list
	# layout
	grid $info.l_0 $info.0 -sticky new -padx 5 -pady 3
	grid $info.l_1 $info.1 -sticky new -padx 5 -pady 3
	grid $info.l_2 $info.2 -sticky new -padx 5 -pady 3
	grid $info.l_3 $info.3 -sticky new -padx 5 -pady 3
	grid $info.l_6 $info.6 -sticky new -padx 5 -pady 3
	grid $info.l_5 $info.5 -sticky new -padx 5 -pady 3

	pack $use.l_7 -side top -padx 5 -pady 3 -fill x
	pack $use.list -side top -expand true -fill both -padx 5 -pady 3
#	grid $use.l_7 -sticky new -padx 5 -pady 3
#	grid $use.list -sticky news -padx 5 -pady 3

	pack $top.i $top.u -side left -expand true -fill both -pady 5
	# wait
	waitforModalDialog
}

proc library::routine:move:check {w x y entry} {
variable data


	set data(t:entry) [$data(w:routine:class) nearest $y]
	if {$data(t:entry) == ""} {
		return
	}
	$data(w:routine:class) anchor set $data(t:entry)
	return "cross"
}

proc library::routine:move {w type sent} {
global status
variable data


	set figure [lindex $sent 0]
	routine:check:figure $figure
	# check if dest == src
	set new [file dirname [string range [$w entrycget $data(t:entry) -data] \
								[expr [string length $status(dir:routine)]+1] end]]
	set old [file dirname $figure]
	if {[string compare $old $new] == 0} {
		return
	}
	# can't move if used in current routine
	if {[lsearch -exact [routine::getFigures .routine] $figure] >= 0} {
		tk_messageBox -icon "error" -type ok -title "Error" \
				-message "This figure is used in the routine you are editing.\nYou can't move it"
		return
	}
	# need some fixing in some routines
	if {[llength $data(t:list)] > 0} {
		if {[tk_messageBox -icon "warning" -type yesno -default yes -title "Error" \
					-message "Moving the figure will change the file(s) :\n\n[join $data(t:list) {, }]\n\nDo you want to continue ?"] == "no"} {
			return
		}
		# move the file & delete entry
		file rename -force -- $status(dir:routine)/$figure $status(dir:routine)/$new
		$data(w:routine) delete entry [lindex $sent 2]
		# update references
		foreach f $data(t:list) {
			routine:move:update $f $figure $new/[file tail $figure]
		}
	}
}

proc library::routine:move:update {filename old new} {
global status

	# read the file
	set in [open $status(dir:routine)/$filename "r"]
	set src ""
	while {![eof $in]} {
		append src [gets $in] "\n"
	}
	close $in
	# replace old path with new one
	regsub -all "routine\\($old\\)" $src "routine($new)" newsrc
	# save new file
	set out [open $status(dir:routine)/$filename "w"]
	puts -nonewline $out $newsrc
	close $out
}


#=================================================================================================
#
#	Helper for management of routine files
#

proc library::routine:check:onedir {what directory} {
global status
variable data

	foreach f [glob -nocomplain $directory/*] {
		if {[file isdirectory $f]} {
			library::routine:check:onedir $what $f
		} elseif {[check $what $f $data(t:checked)]} {
			lappend data(t:list) [string range $f \
										 [expr [string length $status(dir:routine)]+1] end]
		}
	}
}

proc library::routine:check:class {class} {
global status
variable data

	set data(t:list) {}
	set data(t:checked) $class
	routine:check:onedir useclass $status(dir:routine)
	return $data(t:list)
}

proc library::routine:check:category {category} {
global status
variable data

	set data(t:list) {}
	set data(t:checked) $category
	routine:check:onedir usecategory $status(dir:routine)
}

proc library::routine:check:figure {figure} {
global status
variable data

	set data(t:list) {}
	set data(t:checked) $figure
	routine:check:onedir usefigure $status(dir:routine)
}

#-------------------------------------------------------------------------------------------------

proc library::routine:list:usedby {w} {
variable data

	tixScrolledHList $w -options {hlist.columns 2  hlist.header true  hlist.height 5}
	set l [$w subwidget hlist]
	set header [tixDisplayStyle text -refwindow $l \
				-fg black -anchor c -padx 8 -pady 2 -font library:routine:header]
	$l header create 0 -itemtype text -text Filename -style $header -borderwidth 1
	$l header create 1 -itemtype text -text "Name of figure" -style $header -borderwidth 1
	set style [tixDisplayStyle text -refwindow $l -anchor nw -font library:routine:text]
	set r [::routine]
	foreach f [lsort -dictionary $data(t:list)] {
		$r parse $f
		set row [$l addchild {} -text "$f" -style $style]
		$l item create $row 1 -itemtype text -text [$r get name] -style $style
	}
	rename $r {}			;# delete the routine
}


proc library::routine:check:showconflict {msg} {
variable data

	# create the window
	set top [buildModalDialog "Library"]
	# widget = label + listbox + button
	label $top.text -text "The following routines are using the $msg"
	routine:list:usedby $top.list
	pack $top.text $top.list -pady 5 -expand true -fill both
	# wait
	waitforModalDialog
}


#-------------------------------------------------------------------------------------------------
# Check if the given figure doesn't make circular references to the routine currently edited

proc library::routine:iscircular {edited figure} {
global status


	set figures [check figures $status(dir:routine)/$figure]
	if {[lsearch -exact $figures $edited] != -1} {
		return 1
	}

	foreach f $figures {
		if {[routine:iscircular $edited $f]} {
			return 1
		}
	}

	return 0
}
