#ifndef INCLUDED_LIST
#define INCLUDED_LIST

//------------------------------------------------------------------------------------------------
//		GENERIC Double linked lists
//------------------------------------------------------------------------------------------------

struct GenericListNode {
	struct GenericListNode *d_prev, *d_next;
	void *d_data;

	// CREATORS
	GenericListNode(void *data) : d_data(data), d_next(0), d_prev(0) {}
	~GenericListNode() {}
};

//------------------------------------------------------------------------------------------------

class GenericList {
protected:
	// DATA
	GenericListNode *d_head, *d_tail;

	// CREATORS
	GenericList() { d_head = d_tail = 0; }
	~GenericList();

	// MANIPULATORS
	void append(void *obj);
	void append(GenericList *list);
	void prepend(void *obj);
	void prepend(GenericList *list);
	void insert(GenericListNode *position, void *obj);
	void insert(GenericListNode *position, GenericList *list);
	void *destroy(void *obj);
	void *replace(void *oldobj, void *newobj);

private:
	int checkDuplicate(void *obj);
};


//------------------------------------------------------------------------------------------------
//		Double linked lists (unique)
//------------------------------------------------------------------------------------------------

template<class T> class ListIterator;

template<class T> class List : public GenericList {
private:
	// not implemented
	List(const List &);
	List &operator=(const List &);

public:
	// CREATORS 
	List() {}
	~List() {}

	// MANIPULATORS
	void free();

	void append(T *obj) { GenericList::append(obj); }
	void append(List<T> *list) { GenericList::append(list); }
	void prepend(T *obj) { GenericList::prepend(obj); }
	void prepend(List<T> *list) { GenericList::prepend(list); }
	void insert(ListIterator<T> &it, T *obj) { GenericList::insert(it.d_node, obj); }
	void insert(ListIterator<T> &it, List<T> *list) { GenericList::insert(it.d_node, list); }
	void replace(T *oldobj, T *newobj);
	void remove(T *obj) { GenericList::destroy(obj); }
	void destroy(T *obj);

	// ACCESSORS
	GenericListNode *head() const { return d_head; }
	GenericListNode *tail() const { return d_tail; }
	T *first() const { return (T *) d_head->d_data; }
	T *last() const { return (T *) d_tail->d_data; }
	int count() const { int c=1; GenericListNode *tmp = d_head;
						while(tmp) {c++; tmp=tmp->d_next;}
						return c; }
};

//------------------------------------------------------------------------------------------------

template<class T> class ListIterator {
	// FRIENDS
	friend class List<T>;

	// DATA
	struct GenericListNode *d_node;

private:
	// not implemented
	ListIterator &operator=(const ListIterator &);

public:
	// CREATORS
	ListIterator() : d_node(0) {}
	ListIterator(const List<T> &list) : d_node(list.head()) {}
	ListIterator(const List<T> *list) : d_node(list ? list->head() : 0) {}
	ListIterator(const ListIterator<T> &iterator) : d_node(iterator.d_node) {}
	~ListIterator() {}

	// MANIPULATORS
	void operator++() { d_node = d_node->d_next; }
	void operator--() { d_node = d_node->d_prev; }
	void reset(const List<T> &list) { d_node = list.head(); }
	void reset(const List<T> *list) { d_node = (list ? list->head() : 0); }
	void last(const List<T> &list) { d_node = list.tail(); }
	void last(const List<T> *list) { d_node = (list ? list->tail() : 0); }

	// ACCESSORS
	T *operator()() const { return (T *) d_node->d_data; }
	operator const bool() const { return (d_node != 0); }
	int isHead() const { return (d_node->d_prev == 0); }
	int isTail() const { return (d_node->d_next == 0); }
	T *prev() const { return (T *) d_node->d_prev->d_data; }
	T *next() const { return (T *) d_node->d_next->d_data; }
};

#endif
