#ifndef INCLUDED_MOVIE
#define INCLUDED_MOVIE

#include "global.h"

template<class T> class List;


//----------------------------------------------------------------------------------------------- 
//	Bezier3D
//------------------------------------------------------------------------------------------------

class Bezier3D {
	// DATA
	int start, end;

	double startX, startY, startZ;
	double endX, endY, endZ;

	double leftX, leftY, leftZ;
	double rightX, rightY, rightZ;

	// FRIENDS
	friend class ViewMovie;
	friend class Routine; // for the creation of new movies
	friend class Movie; // for saving
	friend int yyparse();

public:
	// CREATORS
	Bezier3D(int s, int e) : start(s), end(e) {}
	~Bezier3D() {}

	// ACCESSORS
	double X(int time) const;
	double Y(int time) const;
	double Z(int time) const;

#ifdef DEBUG
	void Dump(int full = FALSE) const {
		if (full) fprintf(stderr, " (<%g,%g,%g>@%d bezier(<%g,%g,%g>, <%g,%g,%g>) <%g,%g,%g>@%d)",startX,startY,startZ, start, leftX,leftY,leftZ, rightX,rightY,rightZ, endX,endY,endZ, end);
		else fprintf(stderr, " <%g,%g,%g>@%d bezier(<%g,%g,%g>, <%g,%g,%g>)", startX,startY,startZ, start, leftX,leftY,leftZ, rightX,rightY,rightZ); }
	void DumpEnd() const { fprintf(stderr, " <%g,%g,%g>@%d", endX,endY,endZ, end); }
#endif
};


//----------------------------------------------------------------------------------------------- 
//	Movie
//------------------------------------------------------------------------------------------------

class Movie {
	// DATA
	char *d_name;
	class List<Bezier3D> *d_eye, *d_center;

public:
	// CREATORS
	Movie(char *name, List<Bezier3D> *eye, List<Bezier3D> *center);
	~Movie();

	// ACCESSORS
	class List<Bezier3D> *eye() const { return d_eye; }
	class List<Bezier3D> *center() const { return d_center; }
	char *name() const { return d_name; }
	void Save(FILE *out) const;

	// MANIPULATORS
	void NewName(char *name);

#ifdef DEBUG
	void Dump() const;
#endif
};


#endif
