namespace eval routine {
	variable list {}

#	image create pixmap zoomIn -file pixmap/zoomin.xpm
#	image create pixmap zoomOut -file pixmap/zoomout.xpm
#	image create pixmap raz -file pixmap/raz.xpm

#	image create pixmap pose -file pixmap/pose.xpm
#	image create pixmap cog -file pixmap/cog.xpm
#	image create pixmap notes -file pixmap/notes.xpm

	image create pixmap point -file pixmap/point.xpm
	image create pixmap pointgray -file pixmap/pointgray.xpm
	image create pixmap pointred -file pixmap/pointred.xpm
	image create pixmap pointgreen -file pixmap/pointgreen.xpm
	image create pixmap pointblue -file pixmap/pointblue.xpm

	# for toolbars
	image create pixmap tbr_keys -file pixmap/tb_rtn_keys.xpm
	image create pixmap tbr_sticky -file pixmap/tb_rtn_sticky.xpm
	image create pixmap tbr_mod3 -file pixmap/tb_rtn_mod3.xpm
	image create pixmap tbr_mod4 -file pixmap/tb_rtn_mod4.xpm
	image create pixmap tbr_newchar -file pixmap/tb_rtn_newchar.xpm
	image create pixmap tbr_newbeat -file pixmap/tb_rtn_newbeat.xpm
}

proc routine::new {w {pos ""} {size ""}} {
global status tcl_platform
variable list
variable $w
upvar 0 $w data

	if {[winfo exists $w]} {
		raise $w
		return
	}

	# register the editor
	lappend list $w

	# create the window
    set data(w:top) $w
    toplevel $w
	if {$pos != "" || $size != ""} {
	    wm geometry $w $size$pos
	}
    wm title $w "Routine Editor"
	wm protocol $w WM_DELETE_WINDOW "main::closeWindow $w"
	bind $w <Escape> "after idle \"main::closeWindow $w\""

	# the scrollbars & zoom button
	set data(w:vsb)  [scrollbar $w.vsb -takefocus 0]
	set data(w:hlsb) [scrollbar $w.hlsb -orient horizontal -takefocus 0]
	set data(w:hcsb) [scrollbar $w.hcsb -orient horizontal -takefocus 0]

	set data(v:zoom) 1.0
	button $w.zxIn  -bd 1 -padx 0 -pady 0 -image zoomIn -command "routine::zoom $w 1"
	button $w.zxOut -bd 1 -padx 0 -pady 0 -image zoomOut -command "routine::zoom $w -1"
	button $w.zxRaz -bd 1 -padx 0 -pady 0 -image raz -command "routine::zoom $w 0"

	# the hlist
	set data(w:hlist) [tixTree $w.l -scrollbar none -highlightthickness 1 \
							-highlightcolor lightgray -width 110 \
							-opencmd "routine::list:open $w" -closecmd "routine::list:close $w"]
	set data(style:bold) [tixDisplayStyle text -refwindow [$w.l subwidget hlist] \
							-font routine:list:bold]
	set data(style:leaf) [tixDisplayStyle text -refwindow [$w.l subwidget hlist] \
							-fg navyblue -font routine:list]
	$w.l subwidget hlist configure -xscrollcommand "$w.hlsb set" \
			-font routine:list -indicator 0 \
			-yscrollcommand "routine::scrollAdjust $w" -takefocus 0
	$w.hlsb configure -command "[$w.l subwidget hlist] xview"
	$w.vsb configure -command "[$w.l subwidget hlist] yview"

	  # first entry for main comments
	  set hlist [$data(w:hlist) subwidget hlist]
	  $hlist add comment -itemtype text -text "Comments" -style $data(style:bold)

	# the canvas
	set data(w:canvas) [canvas $w.c -bg gray92 -relief sunken -bd 1]
	$w.c configure -xscrollcommand "$w.hcsb set"
	$w.c xview moveto 0
	$w.c yview moveto 0
	$w.hcsb configure -command "$w.c xview"
	bind $w.c <Enter> "[$w.l subwidget hlist] selection clear; \
					   [$w.l subwidget hlist] anchor clear;"
	bind $w.c <Configure> "routine::canvas:resize $w %w %h"
	bind $w.c <ButtonPress-1> "routine::select:init $w %x %y"
	bind $w.c <Button1-Motion> "routine::select:drag $w %x %y"
	bind $w.c <ButtonRelease-1> "routine::select:doIt $w %x %y"
	  # the line to represent the time
	  set id [$w.c create line 0 0 0 1000000 -fill violet -tags time]
	  $w.c bind $id <ButtonPress-1> "set routine::${w}(v:toggle:cursor) 0"
	  $w.c bind $id <Button1-Motion> "routine::moveTime $w %x"
	  $w.c bind $id <ButtonRelease-1> "set routine::${w}(v:toggle:cursor) 1"
	  $w.c bind $id <Enter> "routine::cursor $w sb_h_double_arrow"
	  $w.c bind $id <Leave> "routine::cursor $w {}"

	# the resize handle
	pane::new $w $data(w:hlist) [frame $w.rsz -width 1 -cursor sb_h_double_arrow] $data(w:canvas)

	# the toolbar for canvas
	set tb [toolbar::new $w.toolbar -bd 1]
	  toolbar::add:button $tb new -bd 1 -image tbr_newchar \
			-command "routine::dancer:configure $w -1" -tip "Add a new character"

	bind $tb <Enter> "[$w.l subwidget hlist] selection clear; \
					  [$w.l subwidget hlist] anchor clear;"

	# the toolbar for canvas
	set tb2 [toolbar::new $w.toolbar2 -bd 1]
	  toolbar::add:button $tb2 new -bd 1 -image tbr_newbeat \
			-command "routine::adjustTime $w end 1" -tip "Add a new beat in the routine"
	  toolbar::add:separator $tb2 10 ;#--------------------
	  toolbar::add:check $tb2 keys -bd 1 -image tbr_keys -variable routine::${w}(v:showpartial) \
			-oncommand "$w.c move partial 0 +10000" \
			-offcommand "$w.c move partial 0 -10000" \
			-tip "Show keys in figures"
	  toolbar::add:check $tb2 stick -bd 1 -image tbr_sticky \
			-variable routine::${w}(v:stickyGrid) \
			-tip "Force alignement on (sub)beats"
	  set routine::${w}(v:stickyGrid) 1
	  toolbar::add:separator $tb2 10 ;#--------------------
	  toolbar::add:radio $tb2 mod3 -bd 1 -group rhythm -image tbr_mod3 \
			-tip "Main rhythm in bars of 3 beats" -oncommand "routine::setRhythm $w" \
			-variable routine::${w}(v:rhythm) -value 3000
	  toolbar::add:radio $tb2 mod4 -bd 1 -group rhythm -image tbr_mod4 \
			-tip "Main rhythm in bars of 4 beats" -oncommand "routine::setRhythm $w" \
			-variable routine::${w}(v:rhythm) -value 4000



	# the controls
	set f [frame $w.controls -relief groove -bd 2]
	  # the control to show time
	  set data(w:time) [tixControl $f.time -label "Time " -step 10 \
			-integer true -variable routine::${w}(v:timecontrol) \
			-min 0 -command "routine::sync $w" \
			-options { label.font routine:time entry.font routine:time entry.justify right \
					   entry.borderWidth 1 entry.width 5 entry.highlightColor lightgray}]
		bind $f <Leave> "focus $w"

	  # the player buttons
	  set data(w:player) [frame $f.ff]
	  player::new $f.ff "routine::sync $w"

	  # the new beat button
	  button $f.bnew -text "new b"	-command "routine::adjustTime $w end 1"

	  # layout
	  pack $f.time -side left -padx 10 -pady 5
	  pack $f.ff -side left -padx 10 -pady 5

	# the layout
	grid $data(w:vsb) $data(w:hlist) $w.rsz	$data(w:canvas)	 - - -					-sticky news
	grid x			  $data(w:hlsb)  ^		$w.zxIn	$w.zxOut $data(w:hcsb) $w.zxRaz	-sticky news
	grid $tb		  -				 -		$tb2   			 - - - -sticky news -padx 2 -pady 2
	grid $f        	  - - - - - - -sticky news -padx 2 -pady 5

	grid rowconfig    $w {0} -weight 1
	grid rowconfig    $w {1 2} -weight 0
	grid columnconfig $w {0 1 2 3 4 6} -weight 0
	grid columnconfig $w {5} -weight 1

	# global bindings
	bind KeyRoutine <Key-space> "player::toggle $data(w:player) play"
	if {$tcl_platform(platform) == "windows"} {
		bind KeyRoutine <Mod2-Key-space> "player::toggle $data(w:player) reverse"
	} else {
		bind KeyRoutine <Mod1-Key-space> "player::toggle $data(w:player) reverse"
	}
	bind KeyRoutine <Left> "routine::adjustTime $w -5"
	bind KeyRoutine <Right> "routine::adjustTime $w +5"
	bind KeyRoutine <Shift-Left> "routine::adjustTime $w -1"
	bind KeyRoutine <Shift-Right> "routine::adjustTime $w +1"
	bind KeyRoutine <Up> "routine::adjustTime $w end"
	bind KeyRoutine <Down> "routine::adjustTime $w zero"
	bind KeyRoutine <Insert> "routine::adjustTime $w +20 1"
	bind KeyRoutine <Return> "routine::zoom $w 0"

	bindtags $w "[bindtags $w] KeyRoutine"

	# action menu in canvas
	bind $data(w:canvas) <3> "routine::canvas:actionMenu $w %x %y %X %Y"

	# action menu in hlist
	menu $w.m -tearoff 0 -font menubutton -bd 1 -activeborderwidth 1
	bind [$data(w:hlist) subwidget hlist] <3> "routine::list:actionMenu $w %x %y %X %Y"

	# the drag & drop facility
	drag&drop::target add $data(w:canvas) -type "Routine" -checkcmd "routine::drop:check $w" \
			-cmd "routine::drop:cmd $w" -cleanupcmd "$w.c delete insert"


	# some settings
	set data(v:routine) ""
	set data(v:time) 0
	set data(v:maxtime) 1000
	set data(v:height) 1
	set data(v:width) 0
	set data(v:modified) 0
	set data(v:toggle:cursor) 1
	set	data(v:currentnote) ""
	set routine::${w}(v:rhythm) 3000			;# because of the toolbar
	set routine::${w}(v:showpartial) 0			;# because of the toolbar
#	set data(v:hidden:grid1) 0
#	set data(v:hidden:grid2) 0
#	set data(v:hidden:grid4) 0

	set data(t:done) 0
	set data(t:ok) 0
	set data(t:text) ""
	set data(t:colorCounter) 0

	# body used to view the choices in configuration of body
	set body [body "__configure&new__"]
	set data(v:dancerconfigure:body) $body 
}

proc routine::closeWindow {w} {
variable list
variable $w
upvar 0 $w data

	set i [lsearch -exact $list $w]
	if {$i == -1} { return }
	set list [lreplace $list $i $i]

	# delete the routine & the bodies
	if {[info exists data(v:routine)]} {
		rename $data(v:routine) ""
	}
	if {[info exists data(v:bodies)]} {
		foreach b $data(v:bodies) {
			rename $b ""
		}
	}
	rename $data(v:dancerconfigure:body) ""
	# remove array associated with window
	catch {unset data}
}

#-------------------------------------------------------------------------------------------------

# the name of the part
array set routine::common {
	pose		""
	pose.1		""
	pose.1.0	"head"
	pose.1.1	"torso"
	pose.1.2	"pelvis"
	pose.2		""
	pose.2.1	""
	pose.2.1.1	"left:shoulder"
	pose.2.1.2	"left:elbow"
	pose.2.1.3	"left:wrist"
	pose.2.2	""
	pose.2.2.1	"right:shoulder"
	pose.2.2.2	"right:elbow"
	pose.2.2.3	"right:wrist"
	pose.3		""
	pose.3.1	""
	pose.3.1.1	"left:hip"
	pose.3.1.2	"left:knee"
	pose.3.1.3	"left:ankle"
	pose.3.1.4	"left:foot"
	pose.3.2	""
	pose.3.2.1	"right:hip"
	pose.3.2.2	"right:knee"
	pose.3.2.3	"right:ankle"
	pose.3.2.4	"right:foot"

	cog			""
	cog.1		""
	cog.1.1		"cog:position"
	cog.1.2		"cog:altitude"
	cog.2		""
	cog.2.1		"cog:orientation"
	cog.2.2		"cog:rotX"
	cog.2.3		"cog:rotZ"
}

array set routine::commonReverse {
	head			pose.1.0
	torso			pose.1.1
	pelvis			pose.1.2
	left:shoulder	pose.2.1.1
	left:elbow		pose.2.1.2
	left:wrist		pose.2.1.3
	right:shoulder	pose.2.2.1
	right:elbow		pose.2.2.2
	right:wrist		pose.2.2.3
	left:hip		pose.3.1.1
	left:knee		pose.3.1.2
	left:ankle		pose.3.1.3
	left:foot		pose.3.1.4
	right:hip		pose.3.2.1
	right:knee		pose.3.2.2
	right:ankle		pose.3.2.3
	right:foot		pose.3.2.4
	cog:position	cog.1.1
	cog:altitude	cog.1.2
	cog:orientation	cog.2.1
	cog:rotX		cog.2.2
	cog:rotZ		cog.2.3

	cog				cog.1.1
	cog:r			cog.1.2
}

proc routine::dancer:add {w dancer {fullClose 0}} {
variable $w
upvar 0 $w data

	set hlist [$data(w:hlist) subwidget hlist]

	# add main entry = dancer name
	$hlist add $dancer -itemtype text -text [$data(v:routine) get dancername $dancer] \
			-style $data(style:bold)

	set pose [$hlist add $dancer.pose -itemtype imagetext -text "Pose"]
	set cog [$hlist add $dancer.cog -itemtype imagetext -text "C.o.g."]
	$hlist add $dancer.notes -itemtype imagetext -text "Notes"

	# information about the pose
	foreach {i n} {
		1 "Body"
			1.0 "Head" 1.1 "Torso" 1.2 "Pelvis"
		2 "Arms"
			2.1 "Left arm"
				2.1.1 "Shoulder" 2.1.2 "Elbow" 2.1.3 "Wrist"
			2.2 "Right arm"
				2.2.1 "Shoulder" 2.2.2 "Elbow" 2.2.3 "Wrist"
		3 "Legs"
			3.1 "Left leg"
				3.1.1 "Hip" 3.1.2 "Knee" 3.1.3 "Ankle" 3.1.4 "Foot"
			3.2 "Right leg"
				3.2.1 "Hip" 3.2.2 "Knee" 3.2.3 "Ankle" 3.2.4 "Foot"} {

		$hlist add $pose.$i -itemtype text -text $n -style $data(style:leaf)
	}
	# information about the cog
	foreach {i n} {
		1 "Position"
			1.1 "Floor" 1.2 "Altitude"
		2 "Angles"
			2.1 "Orientation" 2.2 "Roll" 2.3 "Bend"} {

		$hlist add $cog.$i -itemtype text -text $n -style $data(style:leaf)
	}

	# close some entries

	$data(w:hlist) autosetmode
	foreach kid [$hlist info children] {
		$w.l setmode $kid none
	}

	# fill the canvas
	canvas:add:dancer $w $dancer
	# init the tree
	foreach entry {pose.1 pose.2 pose.2.1 pose.2.2 pose.3 pose.3.1 pose.3.2 
				   cog.1 cog.2 } {
		$data(w:hlist) subwidget hlist entryconfigure $dancer.$entry -style {}
	}
	list:close $w $dancer.pose.3.2
	$data(w:hlist) setmode $dancer.pose.3.2 open
	list:close $w $dancer.pose.3.1
	$data(w:hlist) setmode $dancer.pose.3.1 open
	list:close $w $dancer.pose.3
	$data(w:hlist) setmode $dancer.pose.3 open
	list:close $w $dancer.pose.2.2
	$data(w:hlist) setmode $dancer.pose.2.2 open
	list:close $w $dancer.pose.2.1
	$data(w:hlist) setmode $dancer.pose.2.1 open
	list:close $w $dancer.pose.2
	$data(w:hlist) setmode $dancer.pose.2 open
	list:close $w $dancer.pose.1
	$data(w:hlist) setmode $dancer.pose.1 open
	list:close $w $dancer.cog.1
	$data(w:hlist) setmode $dancer.cog.1 open
	list:close $w $dancer.cog.2
	$data(w:hlist) setmode $dancer.cog.2 open

	if {$fullClose} {
		list:close $w $dancer.pose
		$data(w:hlist) setmode $dancer.pose open
		list:close $w $dancer.cog
		$data(w:hlist) setmode $dancer.cog open
	}
}


#=================================================================================================

proc routine::load {w filename} {
variable $w
upvar 0 $w data

	set r [::routine]
	if {$filename != ""} {
		$r load $filename
#puts "loading '$filename' = [time [list $r load $filename]]"
	} else {
		$r dancer add "Character"
		$r set name "Unkown"
		$r set description "To complete"
	}
	wm title $w "Routine Editor <[$r get name]>"
	set data(v:routine) $r
	set data(v:maxtime) 0

	# build main comment
	canvas:add:dancer $w -1

	# build the bodies
	set data(v:bodies) {}
	for {set i 0} {$i<[$r get dancers]} {incr i} {
		# new body with its type
		set body [body::new [$r get dancername $i] [$r get dancertype $i] [$r get dancersex $i]]

		# assign body to routine
		$r set body $i $body
		set data(v:dancer:$body) $i
		dancer:add $w $i
		lappend data(v:bodies) $body
	}

	# setup parameters of the canvas
	incr i -1
	set next [$data(w:canvas) find withtag d$i.notes.line]
	set height [expr int([lindex [$data(w:canvas) coords $next] 1]+16)]
	$data(w:canvas) configure -scrollregion [list -5 0 [expr $data(v:maxtime)+5] $height]
	set data(v:height) $height

	# update windows
	set routine::${w}(v:rhythm) [$r get rhythm] ;# because of the toolbar
												;# setRhythm will create the grid
	$data(w:time) configure -max [$r get beats]
	sync $w 0

	# reset flag for modifs
	set data(v:modified) 0

	# init players
	player::setRoutine $data(w:player) $r
}

#-------------------------------------------------------------------------------------------------

proc routine::save {w filename} {
variable $w
upvar 0 $w data

	$data(v:routine) save $filename
}

#=================================================================================================

proc routine::setTime {w t} {
variable $w
upvar 0 $w data


	set deltaX [expr $t - $data(v:time)]
	if {$deltaX == 0} {
		return
	}
	set data(v:time) $t

	$data(w:canvas) move time [expr $deltaX*$data(v:zoom)] 0
	$data(w:canvas) raise time all
}

proc routine::moveTime {w t} {
variable $w
upvar 0 $w data

	set t [expr int([$data(w:canvas) canvasx $t]/$data(v:zoom))]
	if {$t < 0} {
		set t 0
	}
	if {$t > $data(v:maxtime)} {
		set t $data(v:maxtime)
	}
	sync $w $t
}

#-------------------------------------------------------------------------------------------------

proc routine::adjustTime {w deltaT {new 0}} {
variable $w
upvar 0 $w data

	if {$deltaT == "zero"} {
		set time 0
	} elseif {$deltaT == "end"} {
		set time [$data(v:routine) get beats]
	} else {
		# adjust according to increment
		if {$new} {
			set time [expr ([$data(v:routine) get beats]/50/$deltaT)*$deltaT*50]
		} else {
			set time [expr ([$data(v:routine) get time]/50/$deltaT)*$deltaT*50]
		}
		set time [expr $time + $deltaT*50]
	}
	if {$time < 0} {
		return
	}

	if {$new} {
		# update canvas end & scroll it
		canvas:add:grid $w 1000
		$data(w:canvas) xview moveto 1.0
		# adding a new beat
		incr time 1000
		$data(v:routine) set beats $time
		set data(v:maxtime) $time
		$data(w:time) configure -max $time
		# update viewtracks & movies end
		foreach track $viewTrack::list {
			viewTrack::setMaxTime $track $time
		}		
		# synchro
		sync $w $time
	} elseif {$time <= [$data(v:routine) get beats]} {
		sync $w $time
	}
}

#=================================================================================================

proc routine::update {w cmd body} {
variable $w
upvar 0 $w data
variable commonReverse


	set changes [$data(v:routine) update $cmd $body]
#puts "$data(v:routine) update $cmd $body  ====  changes = $changes"
	foreach {track time} $changes {
		set track $commonReverse($track)

		if {![info exists seen($track:$time)]} {
			# mark joint@time done
			set seen($track:$time) 1
			# find where to put update info
			set dancer $data(v:dancer:$body)
			set y [lindex [$data(w:canvas) coords d$dancer.$track.line] 1]

			# build tag list
			set tags "d$dancer"
			set tag "d$dancer"
			foreach part [split $track "."] {
				append tag ".$part"
				lappend tags $tag
			}

			# update the canvas
#puts "     $track -- $time,[expr abs($time)]  -- y = $y"
			$data(w:canvas) create image [expr abs($time)*$data(v:zoom)] [expr $y+8] \
					-image pointred -tags $tags
		}
	}

	# redisplay the track shown in viewtrack
	foreach t $viewTrack::list {
		viewTrack::redisplay $t
	}

	# set flag for modifs
	set data(v:modified) 1
}

#=================================================================================================

proc routine::drop:check {wdata w x y sent} {
variable $wdata
upvar 0 $wdata data

	$w delete insert

	set x1 [canvasx $wdata $x]
	if {$x1 < 0} { set x1 0 }
	if {$x1 > [expr $data(v:zoom)*$data(v:maxtime)]} {
		set x1 [expr $data(v:zoom)*$data(v:maxtime)]
	}
	set x2 [expr $x1+[expr $data(v:zoom)*[lindex $sent 1]]]
	$w create rectangle $x1 0 $x2 [expr $data(v:height)-1] -outline black -tag insert

	set data(t:dropTime) [expr round($x1/$data(v:zoom))]
	return target
}

proc routine::drop:cmd {wdata w type sent} {
variable $wdata
upvar 0 $wdata data


	figure:insert $wdata [lindex $sent 0]
}

#=================================================================================================

proc routine::sync {w t} {
variable $w
upvar 0 $w data


	set routine::${w}(v:timecontrol) $t		;# because of variable trace
	$data(v:routine) set time $t

	foreach st $stage::list {
		stage::updateDisplay $st
	}
	foreach fedit $figEditor::list {
		figEditor::setTime $fedit
	}
	foreach track $viewTrack::list {
		viewTrack::setTime $track $t
	}
	foreach redit $routine::list {
		routine::setTime $redit $t
	}
}
