#=================================================================================================
#
#		Build & draw the object on canvas
#
#=================================================================================================

proc routine::canvas:add:grid {w nbbeats} {
variable $w
upvar 0 $w data


	# compute start & end for grid
	set start $data(v:maxtime)
	incr data(v:maxtime) $nbbeats
#puts "add:grid  :   start = $start  --  end = $data(v:maxtime)   ----  zoom = $data(v:zoom)   ---   height = $data(v:height)"

	# if start=0 then first call and draw leftmost line
	if {$start == 0} {
		$data(w:canvas) create line 0 0 0 $data(v:height) -fill lightgray -tags grid
	}

	# generate the new lines
	set ids {}
	for {set i $start} {$i < $data(v:maxtime)} {incr i 1000} {
		lappend ids [$data(w:canvas) create line [expr $i+250] 0 [expr $i+250] $data(v:height) \
						-fill lightgray -stipple linedot -tags {grid grid4} ]
		lappend ids [$data(w:canvas) create line [expr $i+750] 0 [expr $i+750] \
							$data(v:height) -fill lightgray -stipple linedot -tags {grid grid4} ]
		lappend ids [$data(w:canvas) create line [expr $i+500] 0 [expr $i+500] \
							$data(v:height) -fill lightgray -stipple linedash -tags {grid grid2} ]
		if {[expr ($i+1000) % $data(v:rhythm)]} {
			set tags {grid grid1}
		} else {
			set tags {grid}
		}
		lappend ids [$data(w:canvas) create line [expr $i+1000] 0 [expr $i+1000] \
							$data(v:height) -fill lightgray -tags $tags ]
	}
	set data(v:hidden:grid1) 0
	set data(v:hidden:grid2) 0
	set data(v:hidden:grid4) 0

	foreach id $ids {
		$data(w:canvas) scale $id 0 0 $data(v:zoom) 1
	}
	$data(w:canvas) lower grid all

	# update the zoom
	zoom $w 2
}

#-------------------------------------------------------------------------------------------------

proc routine::canvas:add:dancer {w dancer} {
variable $w
upvar 0 $w data
variable common


	set c $data(w:canvas)

	# 1. sort list of joint for the right order (because of hash table in Tcl)
	set tmp [array names common pose*]
	set joints [lsort $tmp]
	set tmp [array names common cog*]
	foreach tmp [lsort $tmp] {lappend joints $tmp}

	# 2. where to insert the data
	if {$dancer == -1} {
		set i 0
		set tags ""
	} elseif {$dancer != 0} {
		set next [$c find withtag d[expr $dancer-1].notes.line]
		set i [expr int([lindex [$c coords $next] 1]+16)]
	} else {
		set i 16
	}

	# 3. build the canvas objects if dancer>-1, else main comment
	if {$dancer >= 0} {
		if {$i>0} { $c create line -10 $i 1000000 $i -tags "d$dancer d$dancer.line" }
		incr i 16
		foreach j $joints {
			# 3.1. tag list for this joint
			set tags "d$dancer"
			set tag "d$dancer"
			foreach part [split $j "."] {
				append tag ".$part"
				lappend tags $tag
			}

			if {$common($j) != ""} {
				# 3.2. figures
				foreach r [$data(v:routine) figures $dancer $common($j)] {
					set idx [lindex $r 0]
					set rtags "figure$idx"
					foreach t $tags {
						lappend rtags $t $t.f
					}
					set color [$data(v:routine) figure $idx color]
					set id [$c create rectangle [lindex $r 1] [expr $i+1] \
									[lindex $r 2] [expr $i+15] \
									-tags $rtags -fill $color -outline $color]
					$c lower $id all
					registerTip "[$data(v:routine) figure $idx name]" $c $id
				}

				# 3.3. keys
				set keys [$data(v:routine) keys $dancer $common($j)]
#puts "$common($j)<$dancer> = [lsort -integer $keys]" 
				foreach k $keys {
					if {$k < 0} {
#						$c create rectangle [expr (-$k-3)*$data(v:zoom)] [expr $i+8] \
#									[expr (-$k+3)*$data(v:zoom)] [expr $i+11] \
#									-tags "partial $tags"
						$c create image [expr -$k] [expr $i+8 -10000] \
								-image point -tags "partial $tags"
					} else {
						$c create image $k [expr $i+8] -image pointred -tags "key $tags"
					}
				}
				if {[string match "cog:*" $common($j)]} {
					$c create image -1 [expr $i+8] -image pointgreen -tags $tags
				}
			}

			# 3.4. artificial lines to store info about limits for a joint
			set rtags {}
			foreach t $tags {
				lappend rtags $t $t.f
			}
			$c create line 0 [expr $i+1] 0 [expr $i+1] -tags "$rtags $tag.fmin"
			$c create line 0 [expr $i+15] 0 [expr $i+15] -tags "$rtags $tag.fmax"

			# 3.5. separating line
			$c create line -10 $i 1000000 $i -fill lightgray -tags "$tags $tag.line"
			incr i 16
		}
	}

	# 4. get & show the notes
	# 4.1. real notes
	foreach n [$data(v:routine) keys $dancer notes] {
		# 4.1.1. get the info
		set start [lindex $n 0]
		set end [lindex $n 1]
		set tags "d$dancer d$dancer.notes"
		# 4.1.2. build the objects
#		set ids [$c create image $start [expr $i+8] -image pointblue -tags $tags]
#		lappend ids [$c create line $start [expr $i+8] $end [expr $i+8] \
#					$end [expr $i+5] $end [expr $i+12] -tags $tags]
#		$c raise [lindex $ids 1] grid
#		$c raise [lindex $ids 0]
		set ids [$c create line $start [expr $i+8] $end [expr $i+8] \
					$end [expr $i+5] $end [expr $i+12] -tags $tags]
		lappend ids [$c create image $start [expr $i+8] -image pointblue -tags $tags]
#		$c raise [lindex $ids 1] grid
#		$c raise [lindex $ids 0]
		# 4.1.3. do the bindings
		note:bindings $w [lindex $n 2] $ids
	}
	# 4.2. included notes
	foreach n [$data(v:routine) keys $dancer includednotes] {
		# 4.2.1. get the info
		set start [lindex $n 0]
		set end [lindex $n 1]
		set tags "d$dancer d$dancer.notes"
		# 4.2.2. build the objects
		set ids [$c create image $start [expr $i+8] -image pointgray -tags $tags]
		lappend ids [$c create line $start [expr $i+8] $end [expr $i+8] \
					$end [expr $i+5] $end [expr $i+12] -fill gray -tags $tags]
		$c lower [lindex $ids 0] all
		$c lower [lindex $ids 1] all
		# 4.2.3. do the bindings
		set note [lindex $n 2]
		foreach id $ids {
			$c addtag "note$note" withtag $id
			$c bind $id <Double-1> "routine::note:edit $w $note"
			registerTip {} $c $id "$data(v:routine) note get $note"
		}
	}
	$c create line -10 $i 1000000 $i \
			-fill lightgray -tags "$tags d$dancer.notes.line"

	# 5. show partial if needed
	if {$data(v:showpartial)} {
		$c move partial 0 +10000
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::canvas:add:figure {w routine params apply} {
variable $w
upvar 0 $w data
variable commonReverse


	set c $data(w:canvas)
	set min [expr $data(t:dropTime)*$data(v:zoom)]
	set max [expr ($data(t:dropTime)+[$routine get beats])*$data(v:zoom)]
	set size [expr [$routine get beats]*$data(v:zoom)]

	set figureID [lindex $params 0]
	set dancers [lindex $params 1]
	set joints [lindex $params 2]

	if !$apply {
		foreach dancer $dancers {
			# 1. shift object to make some room for the new figure
			foreach id [$c find withtag d$dancer] {
				if {[expr round([lindex [$c coord $id] 0]/$data(v:zoom))] >= $data(t:dropTime)} {
					$c move $id $size 0
				}
			}
		}
	}

	set index -1
	foreach dancer $data(t:map) {
		incr index
		if {$dancer == -1} {
			continue
		}

		# 2. draw the new figure
		set oldtrack ""
		foreach j [lindex $joints $index] {
			set track $commonReverse($j)
			if {$track != $oldtrack} {
				if {$figureID != -1} {
					# if inserted as figure, draw it
					set ymin [lindex [$c coord d$dancer.$track.fmin] 1]
					set ymax [lindex [$c coord d$dancer.$track.fmax] 1]
					set tags "figure$figureID d$dancer"
					set tag "d$dancer"
					foreach part [split $track "."] {
						append tag ".$part"
						lappend tags $tag $tag.f
					}
					set id [$c create rectangle $min $ymin $max $ymax \
									-tags $tags -fill $data(t:color) -outline $data(t:color)]
					$c lower $id all
					registerTip "[$data(v:routine) figure $figureID name]" $c $id
				}

				# add the keys
				set tags "d$dancer"
				set tag "d$dancer"
				foreach part [split $track "."] {
					append tag ".$part"
					lappend tags $tag
				}
				set y [lindex [$c coord d$dancer.$track.fmin] 1]
				set y [expr int($y/16)*16 + 8]
				set keys [$data(v:routine) keys $dancer $j \
						$data(t:dropTime) [expr $data(t:dropTime)+[$routine get beats]] ]
				foreach k $keys {
					if {$k <0} {
						if {$data(v:showpartial)} {
							$c create image [expr -$k*$data(v:zoom)] $y \
										-image point -tags "partial $tags"
						} else {
							$c create image [expr -$k*$data(v:zoom)] [expr $y -10000] \
										-image point -tags "partial $tags"
						}
					} else {
						$c create image [expr $k*$data(v:zoom)] $y -image pointred \
										-tags "key $tags"
					}
				}
			}
			set oldtrack $track
		}

		# 3. add the notes
		set maxEnd [expr $data(t:dropTime) + [$data(v:routine) get beats]]
		set y [expr [lindex [$c coord d$dancer.notes.line] 1]+8]
		foreach n [$data(v:routine) keys $dancer includednotes] {
			# 3.1. get the info
			set start [lindex $n 0]
			set end [lindex $n 1]
			set tags "d$dancer d$dancer.notes"
			if {$start < $data(t:dropTime) || $end > $maxEnd} {
				continue
			}
			# 3.2. scale coord in canvas
			set start [expr $start*$data(v:zoom)]
			set end [expr $end*$data(v:zoom)]
			# 3.3 build the objects
			set ids [$c create image $start $y -image pointgray -tags $tags]
			lappend ids [$c create line $start $y $end $y $end [expr $y-4] $end [expr $y+4] \
					-fill gray -tags $tags]
			$c lower [lindex $ids 0] all
			$c lower [lindex $ids 1] all
			# 3.3. do the bindings
			set note [lindex $n 2]
			foreach id $ids {
				$c addtag "note$note" withtag $id
				$c bind $id <Double-1> "routine::note:edit $w $note"
				registerTip {} $c $id "$data(v:routine) note get $note"
			}
		}
	}

	# add the new beats in the grid
	set added [$routine get beats]
	if {!$apply} {
		# insertion
		canvas:add:grid $w $added
	} elseif {$data(t:dropTime)+$added > [$data(v:routine) get beats]} {
		# add at the end -> creation of new beats
		set new [expr $data(t:dropTime)+$added-[$data(v:routine) get beats]]
		$data(v:routine) set beats [expr [$data(v:routine) get beats]+$new]
		canvas:add:grid $w $new
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::canvas:checkModif {w dancer track time figureID} {
variable $w
upvar 0 $w data
variable commonReverse


#puts "---------------------------------------------------"
#puts "canvas:checkModif $w $dancer $track $time $figureID"
	if {$figureID == ""} {
		return
	}
	# search for the modif
#puts "    track = $track --->  $commonReverse([string trimright $track "XYZT:"])"
	set track $commonReverse([string trimright $track "XYZT:"])

	set c $data(w:canvas)
	set x1 [expr $time*$data(v:zoom) -2]
	set y1 [lindex [$data(w:canvas) coords d$dancer.$track.line] 1]
	set x2 [expr $x1+4]
	set y2 [expr $y1+15]
#puts "    d$dancer.$track.line  --  $x1,$y1 - $x2,$y2"
	foreach id [$c find overlapping $x1 $y1 $x2 $y2] {
#puts "        id = $id  --  [$c type $id]"
		if {[$c type $id] == "image"} {
#puts "            tags = [$c itemcget $id -tags] -- [string match \"figure$figureID*$track*\" [$c itemcget $id -tags]]"
			if {[string match "figure$figureID*$track*" [$c itemcget $id -tags]]} {
				return
			}
		}
	}

	# not found, so create it
	  # build tag list
	  set tags "key d$dancer"
	  set tag "d$dancer"
	  foreach part [split $track "."] {
		  append tag ".$part"
		  lappend tags $tag
	  }
	  # create the pixmap
	  if {$time==0 && $track == "cog.1.1"} {
	  } else {
		  $c create image [expr $time*$data(v:zoom)] [expr $y1+8] \
				-image pointred -tags $tags
#puts "    pixmap created -- $track ([expr $time*$data(v:zoom)],$y1) @ $time -- tags = $tags"
	  }
}

#-------------------------------------------------------------------------------------------------

proc routine::canvas:resize {w width height} {
variable $w
upvar 0 $w data


	if {$height > $data(v:height)} {
		if {$data(v:hidden:grid1)} { $data(w:canvas) move grid1 0 +10000 }
		if {$data(v:hidden:grid2)} { $data(w:canvas) move grid2 0 +10000 }
		if {$data(v:hidden:grid4)} { $data(w:canvas) move grid4 0 +10000 }
		$data(w:canvas) scale grid 0 0 1 [expr $height/double($data(v:height))]
		if {$data(v:hidden:grid1)} { $data(w:canvas) move grid1 0 -10000 }
		if {$data(v:hidden:grid2)} { $data(w:canvas) move grid2 0 -10000 }
		if {$data(v:hidden:grid4)} { $data(w:canvas) move grid4 0 -10000 }
		set data(v:height) $height
	}

	if {$data(v:width) == 0} {
		zoom $w 0
	} else {
		set data(v:zoom) [expr $data(v:zoom)*$width/double($data(v:width))]
		$data(w:canvas) scale all 0 0 [expr $width/double($data(v:width))] 1
		zoom $w 2
	}
	set data(v:width) $width
}

#-------------------------------------------------------------------------------------------------

proc routine::canvas:offset {w dancer time duration} {
variable $w
upvar 0 $w data


	set c $data(w:canvas)
	set amountX [expr $duration*$data(v:zoom)]
	set limit [expr $time*$data(v:zoom)]

	foreach i [$c find withtag d$dancer] {
		if {[lindex [$c coord $i] 0] >= $limit} {
			# two case :
			#    1. a note is moved as a whole (the point is the reference)
			#    2. everything else is moved alone
			if {[regexp {note([0-9]+)} [$c itemcget $i -tags] {} id]} {
#puts "notes $id -- [$c type $i]"
				if {[$c type $i] == "image"} {
					$c move note$id $amountX 0
				}
			} else {
				$c move $i $amountX 0
			}
		}
	}
}

#=================================================================================================
#
#		Deletion of objects
#
#=================================================================================================

proc routine::delete:tracks {w entry} {
variable $w
upvar 0 $w data
variable common


	scan $entry {%d.%s} dancer track
	delete:keys $w $dancer -1 -1 $track
	# remove items on canvas
	$data(w:canvas) delete "d$entry.f"
	# force update of bodies
	$data(v:routine) set time [$data(v:routine) get time]
}

#-------------------------------------------------------------------------------------------------

proc routine::delete:keys {w dancer start end track} {
variable $w
upvar 0 $w data
variable common


	# delete keys for each joint
	foreach joint [array names common $track*] {
		if {$common($joint) == ""} {
			continue
		}
puts "routine delete $dancer $common($joint) $start $end"
		$data(v:routine) delete $dancer $common($joint) $start $end
	}
	# remove canvas item
	set c $data(w:canvas)
	set x1 [expr ($start-1)*$data(v:zoom)]
	set y1 [lindex [$data(w:canvas) coords d$dancer.$track.line] 1]
	set x2 [expr ($end+1)*$data(v:zoom)]
	set y2 [expr $y1+15]
	foreach id [$c find overlapping $x1 $y1 $x2 $y2] {
		if {[$c type $id]=="image" && [$c itemcget $id -image]=="pointred"} {
			$c delete $id
		}
	}

	# redisplay the track shown in viewtrack
	foreach t $viewTrack::list {
		viewTrack::redisplay $t
	}
}

proc routine::delete:reject {w figureID y} {
variable $w
upvar 0 $w data
variable common

	# get dancer & track
	set part [$data(w:hlist) subwidget hlist nearest $y]
	scan $part {%d.%s} dancer track
#puts "reject -- $figureID -- $dancer -- $track"
	# reject each joint
	foreach joint [array names common $track*] {
		if {$common($joint) == ""} {
			continue
		}
		$data(v:routine) figure $figureID reject $dancer $common($joint)
	}
	# remove the canvas item
	set start [expr [$data(v:routine) figure $figureID start]+1]
	set end [expr [$data(v:routine) figure $figureID end]-1]
	set c $data(w:canvas)
	set x1 [expr ($start-1)*$data(v:zoom)]
	set y1 [lindex [$data(w:canvas) coords d$dancer.$track.line] 1]
	set x2 [expr ($end+1)*$data(v:zoom)]
	set y2 [expr $y1+15]
	foreach id [$c find overlapping $x1 $y1 $x2 $y2] {
		if {[$c type $id]=="image" && [$c itemcget $id -image]=="pointred"} {
			$c delete $id
		} elseif {[$c type $id] == "rectangle"
						&& [string match "figure$figureID*" [$c itemcget $id -tags]]} {
			$c delete $id
		}
	}

	# redisplay the track shown in viewtrack
	foreach t $viewTrack::list {
		viewTrack::redisplay $t
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::delete:dancer {w entry} {
variable $w
upvar 0 $w data

#puts "deletion of dancer $entry"
notImplemented
}

#-------------------------------------------------------------------------------------------------

proc routine::delete:notes {w entry} {
variable $w
upvar 0 $w data
variable common

	if {[scan $entry {%d.notes} dancer] == 0} {
		# comment
#puts "deletion of comments"
notImplemented
	} else {
		# dancer
#puts "deletion of notes $dancer"
notImplemented
	}
}


#=================================================================================================
#
#		Handle the selection of object in the canvas (when the user click)
#
#=================================================================================================

proc routine::select:init {w x y} {
variable $w
upvar 0 $w data

	# check if already processed
#puts "--------- SELECT --------- done = $data(t:done) - ok = $data(t:ok)"
	if {$data(t:done)} {
		return
	}

	set c $data(w:canvas)
	set x1 [expr [$c canvasx $x]-2]
	set y1 [expr int([$c canvasy $y]/16)*16]
	set x2 [expr $x1+4]
	set y2 [expr $y1+15]

	set data(t:ok) 0
	set data(t:sx) [canvasx $w $x]
	set data(t:x) $data(t:sx)
	set data(t:sy) [expr $y1+8]

	foreach id [$c find overlapping $x1 $y1 $x2 $y2] {
		set tags [$c itemcget $id -tags]
#puts "$id = [$c type $id] -- $tags "

#puts "exists [expr round($data(t:sx)/$data(v:zoom))] 0 = [$data(v:routine) note exists [expr round($data(t:sx)/$data(v:zoom))] 0]"

		if {[regexp {.*d(-?[0-9]+)\.notes} $tags {} dancer]
		  && [$data(v:routine) note exists [expr round($data(t:sx)/$data(v:zoom))] $dancer]==-1} {
			#--------------
			# adding a note
			#--------------
			set data(t:type) notes
			set data(t:dancer) $dancer
			set data(t:ok) 1
			set data(t:maxx) [expr [$data(v:routine) note next \
					[expr round($data(t:sx)/$data(v:zoom))] $dancer]*$data(v:zoom)]

			set data(t:ids) {}
#puts "creating canvas notes"
			lappend data(t:ids) [$c create line $data(t:sx) $data(t:sy) $data(t:sx) $data(t:sy) \
					$data(t:sx) [expr $data(t:sy)-5] $data(t:sx) [expr $data(t:sy)+6] \
					-tags "d$dancer d$dancer.notes"]
			lappend data(t:ids) [$c create image $data(t:sx) $data(t:sy) \
					-image pointblue -tags "d$dancer d$dancer.notes"]
			break

		}

#		elseif {[$c type $id] == "line"} {
#			$c itemconfigure $id -fill red
#			after 1000 "$c itemconfigure $id -fill lightgray"
#		}
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::select:drag {w x y} {
variable $w
upvar 0 $w data


#puts "--------- DRAG --------- done = $data(t:done) - ok = $data(t:ok)"
	if {!$data(t:ok)} {
		return
	}

#puts "SELECT drag -- $data(t:type)"
	if {$data(t:type) == "notes"} {
		# notes
		set x [canvasx $w $x]
		if {$x < $data(t:sx)} {
			set x $data(t:sx)
		}
		if {$x > $data(t:maxx)} {
			set x $data(t:maxx)
		}
		$data(w:canvas) coord [lindex $data(t:ids) 0] $data(t:sx) $data(t:sy) $x $data(t:sy) \
				$x [expr $data(t:sy)-5] $x [expr $data(t:sy)+6]
		set data(t:x) $x

	} elseif {$data(t:type) == ""} {
		#
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::select:doIt {w x y} {
variable $w
upvar 0 $w data


#puts "--------- DOIT --------- done = $data(t:done) - ok = $data(t:ok)"
	if {!$data(t:ok) || $data(t:done)} {
		return
	}

	set data(t:done) 0

#puts "SELECT doIt - reset done"
	if {$data(t:type) == "notes"} {
		# ask for the note
		note:edit $w -1

		if {$data(t:text) != ""} {
			# record it within the routine
			set id [$data(v:routine) note add [expr round($data(t:sx)/$data(v:zoom))] \
					[expr round($data(t:x)/$data(v:zoom))] $data(t:dancer) $data(t:text)]
			set data(t:text) ""

			# affect the bindings & add tag to identify it
			note:bindings $w $id $data(t:ids)

			# update movie windows
			foreach m $movie::list {
				movie::sync:notes $m
			}
		} else {
			# destroy the note if no text entered
			$data(w:canvas) delete [lindex $data(t:ids) 0] [lindex $data(t:ids) 1]
		}
		
	} elseif {$data(t:type) == ""} {
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::canvas:actionMenu {w x y X Y} {
variable $w
upvar 0 $w data


	set c $data(w:canvas)
	set x1 [expr [$c canvasx $x]-2]
	set y1 [expr int([$c canvasy $y]/16)*16]
	set x2 [expr $x1+4]
	set y2 [expr $y1+15]

#puts "--------- OBJECT ---------"
	set tmp [$c find overlapping $x1 $y1 $x2 $y2]
	for {set i [expr [llength $tmp]-1]} {$i>=0} {incr i -1} {
		set id [lindex $tmp $i]
		set tags [$c itemcget $id -tags]
#puts "$id = [$c type $id] -- $tags "

		if {[regexp {.*figure(-?[0-9]+).*} $tags {} figureID]} {
			#-------
			# figure
			#-------
			destroy $w.menu
			menu $w.menu -activeborderwidth 1 -bd 1 -font popupmenu -tearoff 0
			  $w.menu add command -label "Reject" \
					  -command "routine::delete:reject $w $figureID $y"
			  figure:mapping:createColorMenu $w $w.menu.m \
					  "routine::figure:changeColor $w $figureID"
			  $w.menu add cascade -label "Color" -menu $w.menu.m
			tk_popup $w.menu $X $Y
			break

		} elseif {[regexp {.*d(-?[0-9]+)\.notes} $tags {} dancer]} {
			#-----
			# note
			#-----
			if {$data(v:currentnote) == ""} {
				return
			}
			destroy $w.menu
			menu $w.menu -activeborderwidth 1 -bd 1 -font popupmenu -tearoff 0
			  $w.menu add command -label "Edit ..." \
					  -command "routine::note:edit $w $data(v:currentnote)"
			  $w.menu add command -label "Delete" \
					  -command "$data(v:routine) note delete $data(v:currentnote); \
								$data(w:canvas) delete note$data(v:currentnote)"

			tk_popup $w.menu $X $Y
			break

		} elseif {[$c type $id]=="image" && [$c itemcget $id -image]=="pointred"} {
			#-----------------------
			# key or modif of figure
			#-----------------------
			set coord [$c coord $id]
			set x [expr round([lindex $coord 0]/$data(v:zoom))]
			set y [expr int([lindex $coord 1])]
#puts "image  -- x=$x  y=$y  --  [$data(w:hlist) subwidget hlist nearest $y]"
			set part [$data(w:hlist) subwidget hlist nearest $y]
			scan $part {%d.%s} dancer track

			destroy $w.menu
			menu $w.menu -activeborderwidth 1 -bd 1 -font popupmenu -tearoff 0
			  $w.menu add command -label "Delete" \
					  -command "routine::delete:keys $w $dancer $x $x $track"

			tk_popup $w.menu $X $Y
			break
		}
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::figure:changeColor {w figureID color} {
variable $w
upvar 0 $w data

	if {$color == {}} {
		set color [tk_chooseColor]
		if {$color == ""} {
			return
		}
	}

	$data(v:routine) figure $figureID color $color
	$data(w:canvas) itemconfigure figure$figureID -fill $color -outline $color
}


#=================================================================================================
#
#		Management of NOTES
#
#=================================================================================================

proc routine::note:bindings {w note ids} {
variable $w
upvar 0 $w data


	set c $data(w:canvas)

	# binding for edition
	foreach i $ids {
		$c addtag "note$note" withtag $i
		$c bind $i <Double-1> "routine::note:edit $w $note"
		registerTip {} $c $i "$data(v:routine) note get $note"
	}
	# bullet
	set obj [lindex $ids 0]
	$c bind $obj <ButtonPress-1> "routine::note:init $w [list $ids] 2 $note"
	$c bind $obj <Button1-Motion> "routine::note:move $w 2 $note %x"
	$c bind $obj <ButtonRelease-1> "routine::note:doIt $w 2 $note"
	$c bind $obj <Enter> "+routine::cursor $w sb_h_double_arrow; \
						  set routine::${w}(v:currentnote) $note"
	$c bind $obj <Leave> "+routine::cursor $w {}; \
						  set routine::${w}(v:currentnote) {}"
	# line
	set obj [lindex $ids 1]
	$c bind $obj <ButtonPress-1> "routine::note:init $w [list $ids] 0 $note"
	$c bind $obj <Button1-Motion> "routine::note:move $w 0 $note %x"
	$c bind $obj <ButtonRelease-1> "routine::note:doIt $w 0 $note"
	$c bind $obj <Enter> "+routine::cursor $w fleur; \
						  set routine::${w}(v:currentnote) $note"
	$c bind $obj <Leave> "+routine::cursor $w {}; \
						  set routine::${w}(v:currentnote) {}"
}

#-------------------------------------------------------------------------------------------------

proc routine::note:dump {w dancer} {
variable $w
upvar 0 $w data


	set time -1
	while {$time <= $data(v:maxtime)} {
		set time [$data(v:routine) note next $time $dancer]
		set end [expr $time+[$data(v:routine) note exists $time $dancer]]
#puts "$time:$end  ----> [$data(v:routine) note text $time $dancer]"
		incr time
	}
}

#-------------------------------------------------------------------------------------------------

proc routine::note:init {w ids what note} {
global status
variable $w
upvar 0 $w data


	# don't show tip while editing note & keep cursor
	set status(v:showtip) 0
	set data(v:toggle:cursor) 0

	# some init
	set data(t:done) 1
	set data(t:ok) 0
	set data(t:moved) 0
	regexp {.*d(-?[0-9]+)\.notes} [$data(w:canvas) itemcget [lindex $ids 0] -tags] {} dancer

#puts "------------------\nINIT $what - set done"
	if {$what == 0} {
		# move
		set data(t:coord) [$data(w:canvas) coord [lindex $ids 0]]
		set data(t:sx) [lindex $data(t:coord) 0]
		set time [expr round($data(t:sx)/$data(v:zoom))]
#puts "$data(v:routine) note prev $time $dancer = [$data(v:routine) note prev $time $dancer]"
#puts "$data(v:routine) note next [expr $time+1] $dancer - $data(v:routine) note exists $time $dancer = [$data(v:routine) note next [expr $time+1] $dancer] - [$data(v:routine) note exists $time $dancer]"
		set data(t:minx) [expr [$data(v:routine) note prev $time $dancer]*$data(v:zoom)]
		set data(t:maxx) [expr ([$data(v:routine) note next [expr $time+1] $dancer]\
				-[$data(v:routine) note exists $time $dancer])*$data(v:zoom)]

	} else {
		# resize
		set data(t:type) notes
		set data(t:ok) 1
		set data(t:ids) $ids
		set data(t:coord) [$data(w:canvas) coord [lindex $ids 0]]
		set data(t:sx) [lindex $data(t:coord) 0]
		set data(t:x) [lindex $data(t:coord) 2]
		set data(t:maxx) [expr [$data(v:routine) note next \
					[expr round($data(t:sx)/$data(v:zoom))+1] $dancer]*$data(v:zoom)]
		set data(t:sy) [lindex $data(t:coord) 1]
	}
}

proc routine::note:move {w what note x} {
variable $w
upvar 0 $w data

	set data(t:moved) 1

	set x [canvasx $w $x]
#puts "NOTE move -- $what"
	if {$what == 0} {
		# move
		if {$x < $data(t:minx)} {
			set x $data(t:minx)
		}
		if {$x > $data(t:maxx)} {
			set x $data(t:maxx)
		}
		$data(w:canvas) move note$note [expr $x-$data(t:sx)] 0
		set data(t:sx) $x
	} else {
		# resize
		select:drag $w $x 0
	}
}

proc routine::note:doIt {w what note} {
global status
variable $w
upvar 0 $w data


	# allows tip & rebind events
	set status(v:showtip) 1
	set data(v:toggle:cursor) 1
	$data(w:canvas) configure -cursor {}

	if {$data(t:moved)} {
		if {$what == 0} {
			$data(v:routine) note modify $note [expr round($data(t:sx)/$data(v:zoom))]
		} else {
			$data(v:routine) note modify $note -1 [expr round($data(t:x)/$data(v:zoom))]
		}
	}
	# break the chain of processing & reset state
	set data(t:ok) 0
	set data(t:done) 0

	# update movie windows
	foreach m $movie::list {
		movie::sync:notes $m
	}
	# update stage windows (for main comment)
	foreach s $stage::list {
		stage::updateDisplay $s
	}
}


#=================================================================================================
#
#		Management of the expansion/collapse of the hierarchical list for dancers/parts
#
#=================================================================================================

proc routine::list:openAll {w entry} {
variable $w
upvar 0 $w data

	if {[$data(w:hlist) getmode $entry] != "open"} {
		return;
	}

	list:open $w $entry
	$data(w:hlist) setmode $entry close

	foreach kid [$data(w:hlist) subwidget hlist info children $entry] {
		list:openAll $w $kid
	}
}

proc routine::list:open {w entry} {
variable $w
upvar 0 $w data

	foreach kid [$data(w:hlist) subwidget hlist info children $entry] {
        $data(w:hlist) subwidget hlist show entry $kid
    }

	# shift the other elements & adjust scrollregion
	list:shift $w $entry [list:expand $w $entry]
}

proc routine::list:closeAll {w entry} {
variable $w
upvar 0 $w data

	if {[$data(w:hlist) getmode $entry] != "close"} {
		return;
	}

	foreach kid [$data(w:hlist) subwidget hlist info children $entry] {
		list:closeAll $w $kid
	}

	list:close $w $entry
	$data(w:hlist) setmode $entry open
}

proc routine::list:close {w entry} {
variable $w
upvar 0 $w data

	foreach kid [$data(w:hlist) subwidget hlist info children $entry] {
		$data(w:hlist) subwidget hlist hide entry $kid
    }

	# shift the other elements & adjust scrollregion
	list:shift $w $entry [list:collapse $w $entry]
}

#-------------------------------------------------------------------------------------------------

proc routine::list:collapse {w entry} {
variable $w
upvar 0 $w data

	set kids [$data(w:hlist) subwidget hlist info children $entry]
	set nbKids [llength $kids]
	set idx 0

	foreach kid $kids {
		set offset 0
		if {[$data(w:hlist) getmode $kid] == "close"} {
			set offset [list:collapse $w $kid]
		}

		# move all the entries 
		set kidY [lindex [$data(w:canvas) coords d$kid.line] 1]
        set deltaY [expr  [lindex [$data(w:canvas) coords d$entry.line] 1] - $kidY]
		$data(w:canvas) move d$kid 0 $deltaY

		# handle special case of figure : resize and move them
		$data(w:canvas) scale d$kid.f 0 [expr $kidY+$deltaY] 1 [expr 1/double($nbKids)]
		$data(w:canvas) move d$kid.f 0 [expr 16.0*$idx/$nbKids]

		incr idx
	}

	return [expr $deltaY+$offset]
}

#-------------------------------------------------------------------------------------------------

proc routine::list:expand {w entry} {
variable $w
upvar 0 $w data

	set kids [$data(w:hlist) subwidget hlist info children $entry]
	set nbKids [llength $kids]
	set idx 0
	set deltaY 0

	foreach kid $kids {
		# handle special case of figure : resize and move them
		if {[$data(w:canvas) coord d$kid.f] != {}} {
			set kidY [lindex [$data(w:canvas) coords d$entry.line] 1]
			$data(w:canvas) move d$kid.f 0 [expr -16.0*$idx/$nbKids]
			$data(w:canvas) scale d$kid.f 0 $kidY 1 $nbKids
		}

		# move all the entries
		incr deltaY 16
		$data(w:canvas) move d$kid 0 $deltaY

		# recursively expand children
		if {[$data(w:hlist) getmode $kid] == "close"} {
			incr deltaY [list:expand $w $kid]
		}

		incr idx
	}

	return $deltaY
}

#-------------------------------------------------------------------------------------------------

proc routine::list:shift {w entry deltaY} {
variable $w
upvar 0 $w data


	set parent [$data(w:hlist) subwidget hlist info parent $entry]

	while {$entry != {}} {
		if {$parent == ""} {
			set siblings [$data(w:hlist) subwidget hlist info children]
		} else {
			set siblings [$data(w:hlist) subwidget hlist info children $parent]
		}
		
		foreach next [lrange $siblings [expr [lsearch $siblings $entry]+1] end] {
			$data(w:canvas) move d$next 0 $deltaY
		}
		set entry $parent
		if {$parent != ""} {
			set parent [$data(w:hlist) subwidget hlist info parent $parent]
		}
	}

	# adjust the canvas scrollregion & the size of the grid
	set list [$data(w:canvas) cget -scrollregion]
	if {$list == {}} {
		return
	}
	set height [expr [lindex $list 3]+$deltaY]
	if {$height > [winfo height $data(w:canvas)]} {
		if {$data(v:hidden:grid1)} { $data(w:canvas) move grid1 0 +10000 }
		if {$data(v:hidden:grid2)} { $data(w:canvas) move grid2 0 +10000 }
		if {$data(v:hidden:grid4)} { $data(w:canvas) move grid4 0 +10000 }
		$data(w:canvas) scale grid 0 0 1 [expr $height/double($data(v:height))]
		if {$data(v:hidden:grid1)} { $data(w:canvas) move grid1 0 -10000 }
		if {$data(v:hidden:grid2)} { $data(w:canvas) move grid2 0 -10000 }
		if {$data(v:hidden:grid4)} { $data(w:canvas) move grid4 0 -10000 }
		set data(v:height) $height
	}
	$data(w:canvas) configure -scrollregion [lreplace $list 3 3 $height]	
}

#=================================================================================================
#
#		Handle the scrolling & the zoom
#
#=================================================================================================

proc routine::scrollAdjust {w arg1 arg2} {
variable $w
upvar 0 $w data

	$data(w:vsb) set $arg1 $arg2
	$data(w:canvas) yview moveto $arg1
}

proc routine::zoom {w incr} {
variable $w
upvar 0 $w data

	if {$incr == 0} {
		set oldzoom $data(v:zoom)
		if {$data(v:maxtime)} {
			set data(v:zoom) [expr ([winfo width $data(w:canvas)]-13.0)/$data(v:maxtime)]
			$data(w:canvas) scale all 0 0 [expr $data(v:zoom)/$oldzoom] 1
		}
	} elseif {$incr == 1} {
		set data(v:zoom) [expr $data(v:zoom)*1.25]
		$data(w:canvas) scale all 0 0 1.25 1
	} elseif {$incr == -1} {
		set data(v:zoom) [expr $data(v:zoom)*.8]
		$data(w:canvas) scale all 0 0 .8 1
	}

	set ratio [expr $data(v:zoom)/15*[winfo width $data(w:canvas)]]
	if {$ratio < 0.5} {
		if {!$data(v:hidden:grid1)} {
			$data(w:canvas) move grid1 0 -10000
		}
		set data(v:hidden:grid1) 1
	} else {
		if {$data(v:hidden:grid1)} {
			$data(w:canvas) move grid1 0 +10000
		}
		set data(v:hidden:grid1) 0
	}
	if {$ratio < 1.0} {
		if {!$data(v:hidden:grid2)} {
			$data(w:canvas) move grid2 0 -10000
		}
		set data(v:hidden:grid2) 1
	} else {
		if {$data(v:hidden:grid2)} {
			$data(w:canvas) move grid2 0 +10000
		}
		set data(v:hidden:grid2) 0
	}
	if {$ratio < 2.0} {
		if {!$data(v:hidden:grid4)} {
			$data(w:canvas) move grid4 0 -10000
		}
		set data(v:hidden:grid4) 1
	} else {
		if {$data(v:hidden:grid4)} {
			$data(w:canvas) move grid4 0 +10000
		}
		set data(v:hidden:grid4) 0
	}

	set list [$data(w:canvas) cget -scrollregion]
	if {$list != {}} {
		set newMax [expr $data(v:maxtime)*$data(v:zoom)+5]
		set list [lreplace $list 2 2 [expr $newMax]]
		$data(w:canvas) configure -scrollregion $list
	}
}
