proc routine::note:edit {w note} {
variable $w
upvar 0 $w data
global status
global __dialog__


	# toplevel
	if {![winfo exists .editnote]} {
		toplevel .editnote
		#wm transient .editnote $w
		wm title .editnote "Note editor"
		wm resizable .editnote 0 0
		wm withdraw .editnote

		# two buttons & a text entry
		set f [frame .editnote.top]
		button $f.ok -default active -bd 1 -image imgOk -command "set __dialog__ 1"
		button $f.nok -bd 1 -image imgCancel -command "set __dialog__ 0"
		tixScrolledText $f.e -scrollbar {auto +y}
		set text [$f.e subwidget text]
		$text configure -width 40 -height 10 -font tip -wrap none
		focus $text
		pack $f.nok $f.ok -side left -anchor w
		pack $f.e -side left -expand true -fill x
		pack $f -padx 3 -pady 3 -expand true -fill x
		# bindings
		bind $text <Escape> "set __dialog__ 0"
		bind $text <Shift-Return> "set __dialog__ 1 ; break"
		bind $text <Control-Return> "set __dialog__ 1 ; break"

		# separator
		set sep [frame .editnote.sep -relief sunken -height 2 -bd 2]
		pack $sep -expand true -fill x -pady 3
		# toolbar for class
		set tb [toolbar::new .editnote.tb -bd 1]
		variable noteClass
		set len -1
		foreach n [array names noteClass] {
			if {[string length $n] > $len} {
				set len [string length $n]
			}
		}
		set i 0
		foreach n [lsort -dictionary [array names noteClass]] {
			toolbar::add:radio $tb $i -bd 1 -group mode -text $n -width $len -font tip \
					-variable routine::${w}(t:noteeditor:mode) \
					-oncommand "routine::note:editor:class .editnote.b"
			incr i
		}

		pack $tb -expand true -fill x -padx 3 -pady 3

		# build helper buttons
		set f [frame .editnote.b]
		note:editor:buttons $w $text $f
		pack $f -fill both -padx 3 -pady 3
	}
	set text [.editnote.top.e subwidget text]

	# show window
	wm geometry .editnote +[expr [winfo pointerx $w]-40]+[expr [winfo pointery $w]+13]
	wm deiconify .editnote
	$text delete 0.0 end
	$text insert 0.0 [$data(v:routine) note get $note]
	# set the class
	if {![info exists data(t:noteeditor:mode)]} {
		set routine::${w}(t:noteeditor:mode) 0
	} else {
		set routine::${w}(t:noteeditor:mode) $data(t:noteeditor:mode)
	}
	# do the dialog
	grab set .editnote
	tkwait variable __dialog__
	grab release .editnote
	wm withdraw .editnote

	if {$__dialog__} {
		set data(t:text) [$text get 0.0 "end - 1 chars"]
		if {$note != -1} {
			$data(v:routine) note modify $note -1 -1 $data(t:text)
			# update movie windows
			foreach m $movie::list {
				movie::sync:notes $m
			}
			# update stage windows (for main comment)
			foreach s $stage::list {
				stage::updateDisplay $s
			}
		}
	} else {
		set data(t:text) ""
	}

}


#=================================================================================================


proc routine::note:editor:buttons {w text top} {
variable noteClass
variable notePanel

	# list of categories & frame for buttons
	tixScrolledHList $top.l -highlightcolor lightgray -options {hlist.width 10 hlist.height 3
																hlist.font tip}
	set l [$top.l subwidget hlist]
	$l configure -browsecmd "routine::note:editor:category $w $top.f $text $l"
	pack $top.l -side left -fill both
	set f [frame $top.f]
	pack $f -side left -anchor w

	# variable to calculate width & height
	set width -1
	set height -1

	# setup the panels
	foreach name [array names notePanel] {
		foreach {entry action content} $notePanel($name) {
			# add category
			set w [frame $f.$name]
			foreach {panel pos buttons} $content {
				# new panel
				set counter 0
				tixLabelFrame $w.[stringify $panel] -padx 1 -pady 1 \
						-label $panel -option {label.font tip label.padX 1}
				set p [$w.[stringify $panel] subwidget frame]
				foreach b $buttons {
					# populate it with buttons
					set len [string length $b]
					if {$len < 5} {
						set len 5
					}
					button $p.$counter -bd 1 -font tip -padx 1 -pady 1 -width $len \
							-text $b -command "routine::note:editor:insert $text [list $b]"
					pack $p.$counter -fill x -padx 2
					incr counter
				}
				grid $w.[stringify $panel] -column [lindex $pos 0] -row [lindex $pos 1] \
						-columnspan [lindex $pos 2] -rowspan [lindex $pos 3] -sticky news
			}
			grid columnconfig $w {0 1 2 3 4 5 6 7 8 9} -weight 1
			grid rowconfig $w {0 1 2 3 4 5 6 7 8 9} -weight 1 

			::update idletasks
			if {$width < [winfo reqwidth $w]} {
				set width [winfo reqwidth $w]
			}
			if {$height < [winfo reqheight $w]} {
				set height [winfo reqheight $w]
			}
		}
	}
	# set size of frame & avoid dynamic resizing
	pack propagate $f 0
	$f configure -width $width -height $height
}

#-------------------------------------------------------------------------------------------------

proc routine::note:editor:class {frame index} {
variable noteClass
variable notePanel

	# update hlist
	set hlist [$frame.l subwidget hlist]
	$hlist delete all
	set name [lindex [lsort -dictionary [array names noteClass]] $index]
	foreach entry $noteClass($name) {
		$hlist add $entry -text [lindex $notePanel($entry) 0] \
			-data [lindex $notePanel($entry) 1]
	}
	# pack the right frame
	pack forget [pack slaves $frame.f]
}


#-------------------------------------------------------------------------------------------------

proc routine::note:editor:category {w frame text list entry} {
variable $w
upvar 0 $w data

	if {[string compare [tixEvent type] "<ButtonRelease-1>"]} {
		return
	}
	# remove old panel & pack new one
	pack forget [pack slaves $frame]
	pack $frame.$entry -anchor w -fill both
	# if there is an action, do it
	set insert [$list entrycget $entry -data]
	if {$insert != ""} {
		note:editor:insert $text $insert 0
	}
	# restore focus to text widget
	focus $text
}

#-------------------------------------------------------------------------------------------------

proc routine::note:editor:insert {text data {space 1}} {

	if {!$space} {
		set line [string trim $data]
		set old [$text get "insert linestart" "insert linestart +[string length $line] chars"]
		if {$line == $old} {
			return
		}
	}
    catch {
   	    if {[$text compare sel.first <= insert]
                && [$text compare sel.last >= insert]} {
       	    $text delete sel.first sel.last
   	    }
    }
	if {[$text compare insert == "insert linestart"]
			|| [$text get "insert - 1 chars" insert]=="\t"
			|| [$text get "insert - 1 chars" insert]=="."} {
		set data [string toupper [string range $data 0 0]][string range $data 1 end]
	}
	if {[$text compare insert == "insert linestart"]
			|| [$text get "insert - 1 chars" insert]=="\t"
			|| [$text get "insert - 1 chars" insert]==" "} {
		$text insert insert "$data"		;# without leading space
	} else {
		$text insert insert " $data"	;# put a space between expressions
	}
}


#=================================================================================================


array set routine::notePanel {

	main {
		Main ""
			{	Beat	{0 0 1 2}	{1, 2, 3, 4, &, a,}
				Foot	{1 0 1 1}	{LF RF}
				Rhythm	{1 1 1 1}	{slow, quick,}
				Direction {2 0 1 2} {forward back "to side" "closes to" "closes towards"
									 across crosses slightly}
				Other	{3 0 1 2}	{C.B.M. C.B.M.P. O.P. P.P. "Counter P.P." in and}
			}
	}

	spball {
		Special ""
			{	Hell	{0 0 1 1}	{"(Heel turn)" "(Heel pull)"}
				Lead	{0 1 1 1}	{"L. side leading" "R. side leading"}
				Action	{1 0 1 2}	{"brushes to" "having brushed"
									 "without weight" "transfert weight back"}
				Foot	{2 0 1 2}	{LF RF}
			}
	}

	splatin {
		Special ""
			{	Direction {0 0 1 1}	{"Left" "Right"}
				Lead	{1 0 1 1}	{"shoulder leading" "preparing to lead"}
				Partner	{0 1 1 1}	{"partner outside" "on Left side" "on Right side" "(Pivot)"}
				Leg		{1 1 1 1}	{"without weight" "replace weight to" "knee flexed"
									 "leg straight" "heel raised"}
			}
	}

	hold {
		Hold ""
			{	Hold	{0 0 1 1}	{"release hold with" "under"}
				Partner	{1 0 1 1}	{"partner outside" "on Left side" "on Right side" "(Pivot)"}
				Part	{2 0 1 2}	{"Left Hand" "Right Hand" "Left Arm" "Right Arm"}
			}
	}

	align {
		Alignement ""
			{	Orientation	{0 0 1 1} {facing backing pointing "pointing down"}
				Direction {1 0 1 1}	{D.W. D.C. LoD "against the"}
			}
	}

	turn {
		Turn "\n\nTurn\t"
			{	Action 	{0 0 1 1}	{"commence to turn" "continue to turn" "complete"}
				Direction {0 1 1 1}	{"to L" "to R"}
				Amount	{1 0 1 2}	{1/8 1/4 3/8 1/2 5/8 3/4}
				When	{2 0 1 2}	{between 1-2 2-3 3-4 4-5}
			}
	}

	rise {
		"Rise & Fall" "\nRise&Fall\t"
			{	What	{0 0 1 1}	{up. down. rise lower "commence to" "continue to"
									 "no foot rise."}
				When 	{1 0 1 1}	{"end of" on and}
				Beat	{2 0 1 1}	{1. 2. 3. 4. 5. 6.}
			}
	}

	ftwrk {
		"Footwork" "\n\nFootwork\t"
			{	Part	{0 0 1 1}	{- Ball Heel Toe "inside edge of" Flat "whole foot"}
				Foot	{1 0 1 1}	{"Left Foot" "Right Foot"}
			}
	}

	sway {
		"Sway" "\nSway\t"
			{	What	{0 0 1 1}	{left right straight}
			}
	}

	pos {
		"Position" ""
			{	Misc	{0 0 1 1}	{"commence in" "position,"}
				Side	{0 1 1 1}	{"Left" "Right"}
				Position {1 0 1 2}	{"closed facing" "close facing" "open facing"
									 "fan" "fallaway" "side-by-side" "contra"}
				Feet 	{2 0 1 2}	{"feet apart," "weight on" "Left Foot" "Right Foot" "back"}
			}
	}
}


array set routine::noteClass {
	Standard	{main spball align turn rise ftwrk sway}
	Latin		{pos main splatin hold align turn ftwrk}
}
