proc routine::getRoutine {w} {
variable $w
upvar 0 $w data

	return $data(v:routine)
}

proc routine::getBodies {w} {
variable $w
upvar 0 $w data

	return $data(v:bodies)
}

proc routine::wasModified {w} {
variable $w
upvar 0 $w data

	return $data(v:modified)
}

proc routine::saved {w} {
variable $w
upvar 0 $w data

	set data(v:modified) 0
}

proc routine::getRhythm {w} {
variable $w
upvar 0 $w data

	return $data(v:rhythm)
}

proc routine::getFigures {w} {
variable $w
upvar 0 $w data

	if [info exists data(v:routine)] {
		return [$data(v:routine) figures]
	} else {
		return {}
	}
}

#=================================================================================================

proc routine::setRhythm {w button} {
variable $w
upvar 0 $w data


	if {![info exists data(v:routine)] || $data(v:routine)==""} {
		return
	}
	# set new rhythm into routine
	$data(v:routine) set rhythm $data(v:rhythm)

	# delete old grid
	$data(w:canvas) delete grid
	# create a new one
	set data(v:maxtime) 0
	canvas:add:grid $w [$data(v:routine) get beats]
	# scale grid & refresh display and hide subdivision
	zoom $w 2
}

#=================================================================================================

proc routine::list:actionMenu {w x y X Y} {
variable $w
upvar 0 $w data


	$w.m delete 0 end

	set hlist [$data(w:hlist) subwidget hlist]
	set entry [$hlist nearest $y]
	$hlist selection clear
	$hlist selection set $entry
	$hlist anchor set $entry


	if {$entry == "comment" || [string match {[0-9].notes} $entry]} {
		# notes
		$w.m add command -label "Editor ..."	;# -command "routine::"
		$w.m add separator
		$w.m add command -label "Delete" -command "routine::delete:notes $w $entry"

	} elseif [string match {[0-9]} $entry] {
		# dancer
		$w.m add command -label "Configure ..." -command "routine::dancer:configure $w $entry"
		$w.m add command -label "Delete" -command "routine::delete:dancer $w $entry"

	} else {
		# tracks
		$w.m add command -label "Collapse all" -command "routine::list:closeAll $w $entry"
		$w.m add command -label "Expand all" -command "routine::list:openAll $w $entry"
		$w.m add separator
		$w.m add command -label "Delete track(s)" -command "routine::delete:tracks $w $entry"
	}

	tk_popup $w.m $X $Y
}


#=================================================================================================
#	Helper functions
#

proc routine::canvasx {w x} {
variable $w
upvar 0 $w data


	if $data(v:stickyGrid) {
		set ratio [expr $data(v:zoom)/15*[winfo width $data(w:canvas)]]
		if {$ratio < 0.5} {
			set grid $data(v:rhythm).0
		} elseif {$ratio < 1.0} {
			set grid 1000.0
		} elseif {$ratio < 2.0} {
			set grid 500.0
		} else {
			set grid 250.0
		}

		set x1 [$data(w:canvas) canvasx $x [expr $data(v:zoom)*$grid]]
	} else {
		set x1 [$data(w:canvas) canvasx $x]
	}

	return $x1
}

#-------------------------------------------------------------------------------------------------

proc routine::cursor {w cursor} {
variable $w
upvar 0 $w data

	if $data(v:toggle:cursor) {
		$data(w:canvas) configure -cursor $cursor
	}
}


#=================================================================================================
#	Dialog to configure dacner & add new dancer
#

proc routine::dancer:configure {w dancer} {
global status
variable $w
upvar 0 $w data

	set top .newdancer
	if [winfo exists $top] {
		raise $top
		return
	}

	# if adding a dancer, check the see if max not exceeded
	if {$dancer == -1 && [$data(v:routine) get dancers]==10} {
		tk_messageBox -icon "error" -title "Error" \
				-message "Too many characters (max is 10)." -type ok
		return
	} 

	# create the window
    toplevel $top
	if {$dancer == -1} {
	    wm title $top "Add character <[$data(v:routine) get name]>"
	} else {
	    wm title $top "Configure character <[$data(v:routine) get name]>"
	}
	wm resizable $top false false
    bind $top <Alt-w> "destroy $top"
    bind $top <Escape> "destroy $top"
	if {$dancer == -1} {
		bind $top <Return> "routine::dancer:new $w"
	} else {
		bind $top <Return> "routine::dancer:configure:OK $w $dancer"
	}

	# edition of the name
	label $top.name -text "Name :"
	entry $top.nameEdit -textvariable "routine::${w}(t:name)"

	# edition of the sex
	label $top.sex -text "Sex :"
	frame $top.sexs
	radiobutton $top.sexs.sexm -text "Man" -value man -variable "routine::${w}(t:sex)" \
			-command "$data(v:dancerconfigure:body) set sex man"
	radiobutton $top.sexs.sexw -text "Woman" -value woman -variable "routine::${w}(t:sex)" \
			-command "$data(v:dancerconfigure:body) set sex woman"
	pack $top.sexs.sexm $top.sexs.sexw -side left

	# edition of the type
	label $top.type -text "Type :"
	set f [frame $top.typelist]
	  togl $f.view -width 100 -height 125 -double true \
							-ident "newdancer" -depth true -class view3D -share $status(opengl)
		$f.view set axis false
		$f.view set floor false
		$f.view set view {-27.0 11.0 0.0 4.12373567275 0.0014512043551 -0.881071361601}
		$f.view add $data(v:dancerconfigure:body)
	  bindtags $f.view "[bindtags $f.view] View3D"
	  tixScrolledHList $f.list -scrollbar "auto" -width 100 -height 140 \
			-highlightcolor lightgray
	  [$f.list subwidget hlist] configure \
			-browsecmd "routine::dancer:configure:typeselect [$f.list subwidget hlist] \
			 	$data(v:dancerconfigure:body)" \
			-command "routine::dancer:configure:typeedit $w $dancer \
				$data(v:dancerconfigure:body)"

	  button $f.new -bd 1 -text "New ..." -command "routine::dancer:configure:typenew $w $dancer"
	  button $f.edit -bd 1 -text "Edit ..." \
			-command "routine::dancer:configure:typeedit $w $dancer $data(v:dancerconfigure:body)"
	  button $f.delete -bd 1 -text "Delete" \
			-command "routine::dancer:configure:typedelete $w $dancer"

	  grid $f.list -		-			$f.view 	-sticky news -padx 5 -pady 5
	  grid $f.new  $f.edit	$f.delete	^			-sticky news -padx 5 -pady 5
	  

	# the button
	set f [frame $top.buttons]
	  button $f.ok -default active -text "OK" -width 5 -underline 0
	  if {$dancer == -1} {
		  $f.ok configure -command "routine::dancer:new $w"
	  } else {
		  $f.ok configure -command "routine::dancer:configure:OK $w $dancer"
	  }
	  button $f.cancel -text "Cancel" -command "destroy $top" -width 5 -underline 0
	  pack $f.ok $f.cancel -side left -padx 10

	# the layout
	grid $top.name		$top.nameEdit	-sticky news -padx 5 -pady 5
	grid $top.sex		$top.sexs		-sticky news -padx 5 -pady 5
	grid $top.type		$top.typelist	-sticky new  -padx 5 -pady 5
	grid $top.buttons	-				-sticky ns -padx 5 -pady 5
	grid configure $top.nameEdit -padx 12
	grid rowconfig    $top {0 2 3 5} -weight 0
	grid rowconfig    $top {1 4} -weight 1
	grid columnconfig $top {0} -weight 0
	grid columnconfig $top {1} -weight 1

	# show shortcuts
	focus $top.nameEdit
	bind $top <Alt-o> "routine::property:OK $w"
	bind $top <Alt-c> "after 100 destroy $top"

	# fill it
	dancer:configure:fill $w $dancer __all__
}

proc routine::dancer:configure:fill {w dancer what} {
variable $w
upvar 0 $w data

	if {$what == "__all__"} {
		if {$dancer == -1} {
			# new dancer
			set data(t:name) "Character"
			set data(t:sex) man
		} else {
			set data(t:name) [$data(v:routine) get dancername $dancer]
			set data(t:sex) [$data(v:routine) get dancersex $dancer]
			set what [$data(v:routine) get dancertype $dancer]
		}
		# apply the sex to body
		$data(v:dancerconfigure:body) set sex $data(t:sex)	
	}

	# types
	set hlist [.newdancer.typelist.list subwidget hlist]
	set i 0
	$hlist delete all
	set types [lsort -dictionary [body::list:types]]
	foreach t $types {
		$hlist add $i -text $t -data $t
		incr i
	}

	# set selection of type (0 by default, or $what else)
	if {$what == "__all__" || $what == ""} {
		set index 0
	} else {
		set index [lsearch -exact $types $what]		
	}
	$hlist selection clear
	$hlist selection set $index
	$hlist anchor set $index
	eval "[$hlist cget -browsecmd] $index"
}

proc routine::dancer:configure:typeselect {w body entry} {
	body::apply:type $body [$w info data $entry]
}

proc routine::dancer:configure:typenew {w dancer} {
	if [catch {set type [body::configure]}] {
		return
	}
	dancer:configure:fill $w $dancer $type
}

proc routine::dancer:configure:typeedit {w dancer body {idx -1}} {
	# get info for the type name
	set hlist [.newdancer.typelist.list subwidget hlist]
	if {$idx == -1} {
		set idx [$hlist info anchor]
	}
	set type [$hlist info data $idx]
	# do it
	if {[catch {set type [body::configure $body $type]}]} {
		return
	}
	dancer:configure:fill $w $dancer $type
}

proc routine::dancer:configure:typedelete {w dancer} {
variable $w
upvar 0 $w data

	set nb [$data(v:routine) get dancers]
	set hlist [.newdancer.typelist.list subwidget hlist]
	set anchor [$hlist info anchor]
	set type [$hlist info data $anchor]

	for {set i 0} {$i < $nb} {incr i} {
		if {$dancer!=$i && [$data(v:routine) get dancertype $i]==$type} {
			tk_messageBox -icon "error" -title "Error" \
					-message "Type is used by other characters" -type ok
			return
		}
	}

	# search the new type to apply
	set new [$hlist info prev $anchor]
	if {$new == {}} {
		set new [$hlist info next $anchor]
	}
	if {$new == {}} {
		tk_messageBox -icon "error" -title "Error" \
				-message "You must keep at least one type !" -type ok
		return
	}
	# remove the type
	set doit [tk_messageBox -icon "question" -type yesno -default yes -title "Error" \
					-message "Do you want really remove the type ?"]
	if {$doit == "no"} {
		return
	}
	body::delete:type $type
	# update window & force body update
	set type [$hlist info data $new]
	dancer:configure:type $w $dancer $type
	dancer:configure:fill $w $dancer $type
}

#-------------------------------------------------------------------------------------------------

proc routine::dancer:configure:OK {w dancer} {
variable $w
upvar 0 $w data

	# name
	dancer:configure:name $w $dancer $data(t:name)
	# sex
	dancer:configure:sex $w $dancer $data(t:sex)
	# type
	set hlist [.newdancer.typelist.list subwidget hlist]
	set type [$hlist info data [$hlist info anchor]]
	dancer:configure:type $w $dancer $type
	# close window
	destroy .newdancer
}

#-------------------------------------------------------------------------------------------------

proc routine::dancer:new {w {interactive 1}} {
variable $w
upvar 0 $w data

	# set index for new dancer
	set i [$data(v:routine) get dancers]
	# create a new body
	if {$interactive} {
		set hlist [.newdancer.typelist.list subwidget hlist]
		set type [$hlist info data [$hlist info anchor]]
	} else {
		set data(t:name) Character
		set data(t:sex) man
		set type ""
	}
	set body [body::new $data(t:name) $type $data(t:sex)]
	# record new dancer in routine
	$data(v:routine) dancer add $data(t:name)
	$data(v:routine) set dancersex $i $data(t:sex)
	$data(v:routine) set dancertype $i $type
	$data(v:routine) set body $i $body
	set data(v:dancer:$body) $i
	dancer:add $w $i
	lappend data(v:bodies) $body
	
	# setup parameters of the canvas
	set next [$data(w:canvas) find withtag d$i.notes.line]
	set height [expr int([lindex [$data(w:canvas) coords $next] 1]+16)]
	$data(w:canvas) configure -scrollregion [list -5 0 [expr $data(v:maxtime)+5] $height]
	$data(w:canvas) scale grid 0 0 1 [expr $height/double($data(v:height))]
	set data(v:height) $height

	# update stage & movie
	stage::add .stage $body
	movie::body:add .movie $i	
	# close window
	destroy .newdancer
}

#-------------------------------------------------------------------------------------------------

proc routine::dancer:configure:name {w dancer name} {
variable $w
upvar 0 $w data

	$data(v:routine) set dancername $dancer $name
	[$data(w:hlist) subwidget hlist] entryconfig $dancer -text $name
	[$data(v:routine) get body $dancer] set name $name

	foreach m $movie::list {
		movie::sync:names $m
	}
	foreach s $stage::list {
		stage::sync:names $s
	}
}

proc routine::dancer:configure:type {w dancer type} {
variable $w
upvar 0 $w data

	$data(v:routine) set dancertype $dancer [body::get:type $type]
	body::apply:type [$data(v:routine) get body $dancer] $type

	# sync & update movie, stage (ghost) windows
	foreach w $movie::list {
		body::apply:type [movie::body:get $w $dancer] $type
	}
	foreach w $stage::list {
		body::apply:type [stage::ghost:get $w $dancer] $type
	}
}

proc routine::dancer:configure:sex {w dancer sex} {
variable $w
upvar 0 $w data

	$data(v:routine) set dancersex $dancer $sex
	[$data(v:routine) get body $dancer] set sex $sex

	# sync & update movie windows
	foreach w $movie::list {
		[movie::body:get $w $dancer] set sex $sex
	}
}


#=================================================================================================
#	Dialog to edit properties of routine (name & description)
#

proc routine::property {w} {
variable $w
upvar 0 $w data

	set top .property
	if [winfo exists $top] {
		property:fill $w
		wm deiconify $top
		raise $top
		return
	}

	# create the window
    toplevel $top
    wm title $top "Properties <[$data(v:routine) get name]>"
    bind $top <Alt-w> "wm withdraw $top"
    bind $top <Escape> "wm withdraw $top"
	bind $top <Return> "routine::property:OK $w"

	# edition of the name
	label $top.name -text "Name :" -underline 0
	entry $top.nameEdit -textvariable "routine::${w}(t:name)" -bd 1

	# edition of the description
	label $top.desc -text "Description :" -underline 0
	tixScrolledText $top.descEdit -scrollbar auto -width 500 -height 100
	[$top.descEdit subwidget text] configure -wrap none

	# edition of the copyright
	label $top.lbpm -text "Bpm :"
	scale $top.bpm -from 20 -to 70 -showvalue true -orient horizontal \
			-variable "routine::${w}(t:bpm)" -bd 1

	# edition of the copyright
	label $top.copy -text "Copyright :" -underline 4
	entry $top.copyEdit -textvariable "routine::${w}(t:copy)" -bd 1

	# edition of the free text
	label $top.text -text "Free text :" -underline 0
	tixScrolledText $top.textEdit -scrollbar auto -width 500 -height 200
	[$top.textEdit subwidget text] configure -wrap none

	# the button
	set f [frame $top.buttons]
	  button $f.ok -default active -text "OK" -width 5 -underline 0 \
			 -command "routine::property:OK $w"
	  button $f.cancel -text "Cancel" -command "wm withdraw $top" -width 5 -underline 0
	  pack $f.ok $f.cancel -side left -padx 10

	# the layout
	grid $top.name		$top.nameEdit	-sticky new -padx 5 -pady 5
	grid $top.desc		$top.descEdit	-sticky new -padx 5 -pady 5
	grid $top.lbpm		$top.bpm		-sticky sew  -padx 5 -pady 5
	grid $top.copy		$top.copyEdit	-sticky new -padx 5 -pady 5
	grid $top.text		$top.textEdit	-sticky new -padx 5 -pady 5
	grid $top.buttons	-				-sticky ns -padx 5 -pady 5
	grid configure    $top.descEdit -sticky news
	grid configure    $top.textEdit -sticky news
	grid rowconfig    $top {0 2 3 5} -weight 0
	grid rowconfig    $top {1 4} -weight 1
	grid columnconfig $top {0} -weight 0
	grid columnconfig $top {1} -weight 1

	# show shortcuts
	focus $top.nameEdit
	bind $top <Alt-n> "focus $top.nameEdit"
	bind $top <Alt-d> "focus [$top.descEdit subwidget text]"
	bind $top <Alt-r> "focus $top.copyEdit"
	bind $top <Alt-f> "focus [$top.textEdit subwidget text]"
	bind $top <Alt-o> "routine::property:OK $w"
	bind $top <Alt-c> "after 100 wm withdraw $top"

	# fill it
	property:fill $w
}

proc routine::property:fill {w} {
variable $w
upvar 0 $w data

	# name
	set data(t:name) [$data(v:routine) get name]
	# description
	[.property.descEdit subwidget text] delete 0.0 end
	[.property.descEdit subwidget text] insert 0.0 [$data(v:routine) get description]
	# bpm
	set data(t:bpm) [$data(v:routine) get bpm]
	# copyright
	set data(t:copy) [$data(v:routine) get copyright]
	# free text
	[.property.textEdit subwidget text] delete 0.0 end
	[.property.textEdit subwidget text] insert 0.0 [$data(v:routine) get text]
}


proc routine::property:OK {w} {
variable $w
upvar 0 $w data

	# get the datas
	$data(v:routine) set name $data(t:name)

	$data(v:routine) set bpm $data(t:bpm)

	set desc [[.property.descEdit subwidget text] get 0.0 "end - 1 chars"]
	$data(v:routine) set description $desc
	$data(v:routine) set copyright $data(t:copy)

	set text [[.property.textEdit subwidget text] get 0.0 "end - 1 chars"]
	$data(v:routine) set text $text

	# destroy the dialog
	after 100 destroy .property

	# set flag for modifs
	set data(v:modified) 1

	# update the title of some windows
	wm title .routine "Routine Editor <$data(t:name)>"
	wm title .stage "Stage <$data(t:name)>"
}
